/**********************************************************************/
/* SimCell: Cell/B.E. Processor Simulator        Arch Lab. TOKYO TECH */
/**********************************************************************/
#include "define.h"

#define IMAGE_HEADER "SimCell_PPE_MEMORY_IMAGE_FILE_VERSION_1.0"
#define LOG_HEADER   "SimCell_LS_LOG_FILE_VERSION_1.0\n"

/**********************************************************************/
ullint gettime()
{
    static ullint start = 0;
    ullint tm;
    struct timeval tv;

    gettimeofday(&tv, NULL);
    tm = tv.tv_sec * 1000000ull + tv.tv_usec;
    if (start == 0)
        start = tm;
    return tm - start;
}

/**********************************************************************/
char *getlinehead(char *dest, FILE *fp)
{
    char *result = fgets(dest, 64, fp);
    if (dest[strlen(dest) - 1] != '\n') {
        char poi[64];
        do
            if (fgets(poi, 64, fp) == NULL)
                break;
        while (poi[strlen(poi) - 1] != '\n');
    }
    return result;
}

/**********************************************************************/
void readmemimage(Chip *chip, uint_032t *map_addr)
{
    FILE *fp;
    char line[65];
    int linecnt = 1;
    char *endptr;
    int regspe = 0, regnum = 0;
    char op[4];
    uint_032t arg1, arg2;

    MainMemory *mem = chip->mem;
    Spe **spe = chip->spe;

    line[64] = op[3] = '\0';

    if (chip->mem_imagefile == NULL)
        return;
    if ((fp = fopen(chip->mem_imagefile, "r")) == NULL) {
        fprintf(stderr, "## can't open file: %s\n", chip->mem_imagefile);
        exit(1);
    }
    getlinehead(line, fp);
    if (strncmp(line, IMAGE_HEADER, strlen(IMAGE_HEADER)) != 0) {
        fprintf(stderr, "## not a memory image file: %s\n",
                chip->mem_imagefile);
        exit(1);
    }

    while (getlinehead(line, fp) != NULL) {
        linecnt++;
        // read a line from file
        if (line[0] != '@')
            continue;
        memcpy(op, &line[1], 3);
        arg1 = strtoul(&line[5], &endptr, 16);
        if (endptr)
            arg2 = strtoul(&endptr[1], NULL, 16);
        else
            arg2 = 0;
        if (strcmp(op, "rgi") == 0) {
            regspe = 0;
            if (arg1 > (uint) chip->spe_num) {
                fprintf(stderr, "## %s:%d: invalid SPE number. skip.\n",
                        chip->mem_imagefile, linecnt);
                continue;
            } else if (arg2 >= NREG) {
                fprintf(stderr, "## %s:%d: too big register number. skip.\n",
                        chip->mem_imagefile, linecnt);
                continue;
            }
            regspe = arg1;
            regnum = arg2;
        } else if (strcmp(op, "rgv") == 0) {
            if (!regspe) {
                fprintf(stderr, "## %s:%d: use @rgi first. skip.",
                        chip->mem_imagefile, linecnt);
                continue;
            }
            spe[regspe - 1]->as->reg[regnum]->put032t(0, arg1);
            spe[regspe - 1]->as->reg[regnum]->put032t(1, arg2);
            regspe = 0;
        } else if (strcmp(op, "mal") == 0) {
            if ((arg1 > (uint) chip->spe_num) || (arg1 == 0)) {
                fprintf(stderr, "## %s:%d: invalid SPE number. skip.\n",
                        chip->mem_imagefile, linecnt);
                continue;
            }
            spe[arg1 - 1]->chi->mfcwritech(29, arg2);
        } else if (strncmp(op, "map", 3) == 0) {
            if ((arg1 > (uint) chip->spe_num) || (arg1 == 0)) {
                fprintf(stderr, "## %s:%d: invalid SPE number. skip.\n",
                        chip->mem_imagefile, linecnt);
                continue;
            }
            map_addr[arg1 - 1] = arg2;
        } else if (strncmp(op, "mem", 3) == 0) {
            mem->write4b(arg1, arg2);
        } else if (strncmp(op, "end", 3) == 0) {
            break;
        } else {
            fprintf(stderr, "## %s:%d: undefined command. skip.",
                    chip->mem_imagefile, linecnt);
        }
    }
    fclose(fp);
}

/**********************************************************************/
void verbose(Chip *chip)
{
    static ullint comma_count = 0;
    ullint inst_count = 0;
    Spe **spe = chip->spe;

    for (int i = 0; i < chip->spe_num; i++)
        inst_count += spe[i]->ss->instcnt;

    inst_count /= chip->verb_cycle;
    for (; comma_count < inst_count; comma_count++) {
        printf(".");
        fflush(stdout);
    }
}

/**********************************************************************/
void cyclelog(Chip *chip)
{
    Spe **spe = chip->spe;

    if (chip->debug_mode == 3) {
        printf("[[cycle %d]]\n", (int) chip->cycle);
        for (int i = 0; i < chip->spe_num; i++) {
            printf("[SPU %d]\n", i + 1);
            spe[i]->spu->printregall();
            printf("pc              0x%x\n\n",
                   (uint) (spe[i]->as->pc - 4));
        }
    }

    if ((chip->log_mode) && (chip->cycle % chip->log_cycle == 0))
        chip->writelogdata();

    if ((chip->cycle % 0x1000 == 0) && (chip->verb_cycle))
        verbose(chip);
}

/**********************************************************************/
ullint atoi_postfix(const char *nptr)
{
    char *endptr;
    ullint result = strtoll(nptr, &endptr, 10);
    if ((*endptr == 'k') || (*endptr == 'K'))
        result *= 1000;
    else if ((*endptr == 'm') || (*endptr == 'M'))
        result *= 1000000;
    else if ((*endptr == 'g') || (*endptr == 'G'))
        result *= 1000000000;
    return result;
}

/**********************************************************************/
Chip::Chip()
{
    cycle = 0;
    max_cycle = MAX_CYCLE_DEF;
    verb_cycle = 0;
    log_cycle = 0;
    running_core = 0;
    spe_num = 1;
    mem_rddelay = MEM_RDDELAY_DEF;
    mem_wrdelay = MEM_WRDELAY_DEF;
    mem_width = MEM_WIDTH_DEF;
    c2c_delay = C2C_DELAY_DEF;
    c2c_width = C2C_WIDTH_DEF;
    log_mode = 0;
    logpage = 0;
    logdata = NULL;
    logfile = NULL;
    mem_imagefile = NULL;
    debug_mode = 0;
    force_mode = 0;
    big_endian = checkendian();
}

/**********************************************************************/
bool Chip::checkendian()
{
    union {
        unsigned short sh;
        unsigned char ch[2];
    } test;
    test.sh = 0xfeff;
    return (test.ch[0] == 0xfe); 
}

/**********************************************************************/
void Chip::setlogmode() 
{
    if (log_mode)
        return;

    log_mode = 1;
    if ((logfile = fopen(LOGMODE_FILE, "wb")) == NULL) {
        fprintf(stderr, "can't open file: %s\n", LOGMODE_FILE);
        exit(1);
    }
    fprintf(logfile, LOG_HEADER);
    fwrite(&spe_num, sizeof(int), 1, logfile);
    fseek(logfile, 64, SEEK_SET);
}

/**********************************************************************/
void Chip::initlogmode()
{
    logdata = new int **[spe_num];
    for (int i = 0; i < spe_num; i++) {
        logdata[i] = new int *[MEM_ACCESS_KIND];
        for (int j = 0; j < MEM_ACCESS_KIND; j++) {
            logdata[i][j] = new int[LS_SIZE >> 4];
            for (int k = 0; k < LS_SIZE >> 4; k++)
                logdata[i][j][k] = 0;
        }
    }
}

/**********************************************************************/
void Chip::setlogdata(int spe, int addr, int type)
{
    spe--;
    addr = addr >> 4;
    logdata[spe][type][addr]++;
}

/**********************************************************************/
void Chip::writelogdata()
{
    char exist[MEM_ACCESS_KIND][LS_SIZE >> 12];
    char data[0x100];
    
    for (int spe = 0; spe < spe_num; spe++) {
        for (int i = 0; i < MEM_ACCESS_KIND; i++) {
            for (int block = 0; block < LS_SIZE >> 12; block++) {
                int temp = 0;
                int off = block << 8;
                for (int j = 0; j < 0x100; j++)
                    temp |= logdata[spe][i][off + j];
                exist[i][block] = (temp != 0);
            }
        }
        fwrite(exist, sizeof(exist), 1, logfile);
        for (int i = 0; i < MEM_ACCESS_KIND; i++) {
            for (int block = 0; block < LS_SIZE >> 12; block++) {
                if (exist[i][block]) {
                    int off = block << 8;
                    for (int j = 0; j < 0x100; j++)
                        data[j] = ((logdata[spe][i][off + j] < 0x100) ?
                                   logdata[spe][i][off + j] : 0xff);
                    fwrite(data, sizeof(data), 1, logfile);
                }
            }
        }
        for (int i = 0; i < LS_SIZE >> 4; i++)
            logdata[spe][MEM_DMA][i] = 0;
    }
    logpage++;
}

/**********************************************************************/
void Chip::finilogmode()
{
    fseek(logfile, 36, SEEK_SET);
    fwrite(&logpage, sizeof(int), 1, logfile);
    fclose(logfile);
    for (int i = 0; i < spe_num; i++) {
        for (int j = 0; j < MEM_ACCESS_KIND; j++) {
            delete[] logdata[i][j];
            logdata[i][j] = NULL;
        }
        delete[] logdata[i];
        logdata[i] = NULL;
    }
    delete[] logdata;
    logdata = NULL;
    logfile = NULL;
    log_mode = 0;
}

/**********************************************************************/
