/**********************************************************************/
/* SimCell: Cell/B.E. Processor Simulator        Arch Lab. TOKYO TECH */
/**********************************************************************/
#include "define.h"

enum {
    LOOP_RUN,
    LOOP_EXIT,
};

/**********************************************************************/
void usage(void)
{
    char usagemessage[] = "\
  -f: ignore errors due to DMA address violation\n\
  -m [filename]: specify the memory image file\n\
  -e[num][kmg]: stop simulation after num cycles executed\n\
  -l[num][kmg]: log mode, writes memory access log to log.dat\n\
  -v[num][kmg]: verbose mode, prints '.' by num\n\
  -n[num]: specify the number of SPEs\n\
  -K[num]: specify the memory read latency\n\
  -L[num]: specify the memory write latency\n\
  -M[num]: specify the memory bandwidth\n\
  -N[num]: specify the core to core latency\n\
  -O[num]: specify the core to core bandwidth\n\
  -d[num]: specify the mode level of debug mode\n\
  e.g. SimCell -n1 -K300 -M30 -m mem006.txt spe006\n";

    printf("Usage: SimCell [-option] object_file_name \n");
    printf("%s", usagemessage);
}

/**********************************************************************/
char *checkarg(Chip *chip, int argc, char **argv)
{
    printf("%s %s\n", L_NAME, L_VER);
    uint num;
    int bin_index = -1;

    for (int i = 1; argv[i] != NULL; i++) {
        char *opt = argv[i];
        if (opt[0] != '-') {
            if (bin_index != -1) {
                fprintf(stderr, "## can't handle multiple binary files\n");
                exit(1);
            }
            bin_index = i;
            continue;
        }
        switch (opt[1]) {
        case 'n':
            num = atoi(&opt[2]);
            if (num > MAX_SPES) {
                fprintf(stderr, "## too many SPEs\n");
                exit(1);
            }
            chip->spe_num = num;
            break;
        case 'd':
            num = atoi(&opt[2]);
            if (num > MAX_DEBUG_MODE) {
                fprintf(stderr, "## debug mode %d not available\n", num);
                exit(1);
            }
            chip->debug_mode = num;
            break;
        case 'e':
            chip->max_cycle = atoi_postfix(&opt[2]);
            if (!chip->max_cycle)
                chip->max_cycle = MAX_CYCLE_DEF;
            break;
        case 'l':
            chip->log_cycle = atoi_postfix(&opt[2]);
            if (!chip->log_cycle)
                chip->log_cycle = LOGMODE_INT_DEF;
            chip->setlogmode();
            break;
        case 'v':
            chip->verb_cycle = atoi_postfix(&opt[2]);
            if (!chip->verb_cycle)
                chip->verb_cycle = VERBOSE_INT_DEF;
            break;
        case 'f':
            chip->force_mode = 1;
            break;
        case 'K':
            chip->mem_rddelay = atoi(&opt[2]);
            break;
        case 'L':
            chip->mem_wrdelay = atoi(&opt[2]);
            break;
        case 'M':
            chip->mem_width = atoi(&opt[2]);
            break;
        case 'N':
            chip->c2c_delay = atoi(&opt[2]);
            break;
        case 'O':
            chip->c2c_width = atoi(&opt[2]);
            break;
        case 'm':
            opt = argv[++i];
            if (opt == NULL) {
                fprintf(stderr, "## image file is not specified\n");
                exit(1);
            }
            chip->mem_imagefile = opt;
            break;
        default:
            fprintf(stderr, "## -%c: wrong option!\n", opt[1]);
            usage();
            exit(1);
        }
    }
    if (bin_index == -1) {
        usage();
        exit(1);
    }

    printf("## SPEs: %d\n", chip->spe_num);
    printf("## Mem Latency R/W, BW:%4d/%4d,%4d | ", 
           chip->mem_rddelay, chip->mem_wrdelay, chip->mem_width);
    printf("C2C Latency, BW:%4d,%4d\n", 
           chip->c2c_delay, chip->c2c_width);
    if (chip->debug_mode)
        printf("## debug mode %d.\n", chip->debug_mode);

    return argv[bin_index];
}

/**********************************************************************/
int speinit(Chip *chip, char *filename, Spe *spe)
{
    Loaderbe *ld = new Loaderbe();

    /*** load and check file ***/
    if (ld->loadelf(filename)) {
        delete ld;
        return 1;
    }

    if (ld->archtype != EM_SPU) {
        fprintf(stderr, "## incorrect architecture : %d\n",
                ld->archtype);
        delete ld;
        return 1;
    }
    if (ld->filetype != ET_EXEC) {
        fprintf(stderr, "## this file isn't executable : %s\n",
                filename);
        delete ld;
        return 1;
    }

    /*** write program to memory ***/
    spe->as->pc = ld->entry;

    for (int i = 0; ld->md[i].id != 0; i++) {
        if (ld->md[i].id == 4)
            spe->ls->write4b((uint_032t) ld->md[i].addr,
                             (uint_032t) ld->md[i].data);
        else
            spe->ls->write4b((uint_032t) ld->md[i].addr,
                             (uint_032t) ld->md[i].data &
                             (0xffffffff << (32 - ld->md[i].id * 8)));
    }

    delete ld;
    ld = NULL;
    return 0;
}

/**********************************************************************/
void initialize(Chip *chip, char *filename)
{
    Mmu *mmu = chip->mmu;
    Spe **spe = chip->spe;

    for (int i = 0; i < chip->spe_num; i++) {
        if (speinit(chip, filename, spe[i])) {
            fprintf(stderr, "Program load Error\n");
            exit(1);
        }
    }

    if (chip->log_mode)
        chip->initlogmode();

    uint_032t map_addr[chip->spe_num];
    for (int i = 0; i < chip->spe_num; i++)
        map_addr[i] = 0x80000000 + LS_SIZE * i;
    readmemimage(chip, map_addr);

    for (int i = 0; i < chip->spe_num; i++)
        mmu->mapls(map_addr[i], i + 1);

    for (int i = 0; i < chip->spe_num; i++)
        spe[i]->spu->start();

    chip->running_core = chip->spe_num;

    gettime();                  // reset elapsed time
}

/**********************************************************************/
void finalize(Chip *chip)
{
    ullint sim_time = gettime();
    MainMemory *mem = chip->mem;
    Spe **spe = chip->spe;

    /*** print result ***/
    if (chip->debug_mode == 0) {
        if (chip->cycle < chip->max_cycle)
            printf("\n## All SPEs are stopped.\n");
        else
            printf("\n## Execution Cycles reached the limit.\n");
    }

    ullint inst_count = 0;
    for (int i = 0; i < chip->spe_num; i++)
        inst_count += spe[i]->ss->instcnt;
    
    if (chip->debug_mode == 1) {
        for (int i = 0; i < chip->spe_num; i++) {
            printf("\n[[SPE %d]]\n", i);
            printf("execute instruction : %10llu\n", 
                   spe[i]->ss->instcnt);
            spe[i]->spu->printregall();
            spe[i]->ls->print();
        }
        mem->print();
    }

    if (chip->debug_mode == 0) {
        printf("## SPEs:%3d | Cycle:%11llu | Inst:%11llu | \n",
               chip->spe_num, chip->cycle, inst_count);
        printf("## Execution  Time:%10.4f | Freq:%7.3f\n",
               (double)chip->cycle / 
               (double)(PROCESSOR_FREQ * AVERAGE_IPC * 1000000),
               (double) PROCESSOR_FREQ / 1000.0);
        printf("## Simulation Time:%10.4f | MIPS:%7.3f\n",
               (double) sim_time / 1000000,
               (double) inst_count / sim_time);
    }

    /*** free when log-mode ***/
    if (chip->log_mode == 1) {
        chip->writelogdata();
        chip->finilogmode();
    }
}

/**********************************************************************/
int loopcond(Chip * chip)
{
    if (chip->cycle >= chip->max_cycle)
        return LOOP_EXIT;

    if (chip->running_core == 0)
        return LOOP_EXIT;

    return LOOP_RUN;
}

/**********************************************************************/
int main(int argc, char **argv)
{
    Chip *chip = new Chip();
    Spe *spe[MAX_SPES];
    new MainMemory(chip);
    new Mmu(chip);
    new SpuInstinfo(chip);

    char *bin_name = checkarg(chip, argc, argv);

    for (int i = 0; i < chip->spe_num; i++)
        spe[i] = new Spe(chip, i + 1);
    Eib *eib = new Eib(chip);

    initialize(chip, bin_name);

    while (loopcond(chip) == LOOP_RUN) {
        chip->cycle++;
        for (int i = 0; i < chip->spe_num; i++) {
            spe[i]->spu->step();
            spe[i]->mfc->step();
            
        }
        eib->step();
        cyclelog(chip);
    }

    finalize(chip);
    return 0;
}

/**********************************************************************/
