/**********************************************************************/
/*  SimCell: Simulator of Cell/B.E.              Arch Lab. TOKYO TECH */
/**********************************************************************/
#include <stdio.h>
#include "define.h"

/**********************************************************************/
enum {
    PSLOT = 0,

    // reason for SPE stop
    SR_NORMALEXIT = 0x2000,
    SR_FUNCCALL   = 0x2100,
    SR_SIGNALMASK = 0xffff,
    SR_STOP       = 0x10000,
    SR_HALT       = 0x20000,
    SR_ERROR      = 0x40000,

    // reason for SPE stall
    ST_NOTRDCH = 0x100,
    ST_NOTWRCH = 0x200,
    ST_CHMASK = 0x1f,
};

/**********************************************************************/
typedef union {
    uint_064t lli;
    double dbl;
} united_064t;

/**********************************************************************/
#define ulow16(x) ((x) & 0xffff)
#define low16(x) exts32(ulow16(x), 16)
#define uhigh16(x) ((x) >> 16)
#define high16(x) exts32(uhigh16(x), 16)
#define slot(x, y) ((chip->big_endian) ? (x) : (y) - 1 - (x))    

/**********************************************************************/
uint_032t exts32(uint_032t x, int y) 
{
    if (y == 32)
        return x;
    uint_032t temp = 0xffffffff << y;
    if (x & (1 << (y - 1)))
        return (temp | (x & ~temp));
    
    else
        return (x & ~temp);
}

/**********************************************************************/
Spu::Spu(Chip *chip, Spe *spe)
{
    this->chip = chip;
    this->spe = spe;
    this->as = spe->as;
    this->ss = spe->ss;
    this->chi = spe->chi;
    inst = new SpuInst(chip, spe);
}

/**********************************************************************/
Spu::~Spu()
{
    delete inst;
    inst = NULL;
}

/**********************************************************************/
void Spu::printregall()
{
    int flag = 0;
    if (chip->debug_mode != 3)
        printf("[SPU Register]\n");
    
    for (int i = 0; i < NREG; i++) {
        if ((as->reg[i]->get064t(0) | as->reg[i]->get064t(1)) == 0) {
            flag++;
            if (flag == 2)
                printf("...\n");
        } else if (flag) {
            flag = 0;
        }
        if (flag < 2) {
            printf("r%3d            uint128 = ", i);
            as->reg[i]->print();
        }
    }
}

/**********************************************************************/
void Spu::start()
{
    as->state = SPE_RUN;
}

/**********************************************************************/
void Spu::step()
{
    if (checkstate() != SPE_RUN)
        return;
    inst->clear();
    fetch(inst);
    decode(inst);
    execute(inst);
    if (inst->nextstate != SPE_STALL) {
        lsaccess(inst);
        writeback(inst);
        ss->instcnt++;
    }
    setnextpc(inst);
    as->state = inst->nextstate;
}

/**********************************************************************/ 
inline int Spu::checkstate()
{
    if (as->state == SPE_STALL) {
        int ca = ss->stall_reason & ST_CHMASK;
        if (ss->stall_reason & ST_NOTRDCH) {
            if (spe->chi->spureadable(ca))
                start();
        }
        else if (ss->stall_reason & ST_NOTWRCH) {
            if (spe->chi->spuwritable(ca))
                start();
        }
    }
    if (as->state == SPE_STOP) {
        if (as->stop_reason == (SR_STOP | SR_FUNCCALL)) {
            spefunccall(spe);
            start();
        }
        else {
            chip->running_core--;
            as->state = SPE_HALT;
        }
    }
    return as->state;
}

/**********************************************************************/ 
inline void Spu::fetch(SpuInst *inst)
{
    spe->ls->read4b(as->pc, &inst->ir);
    inst->pc = as->pc;
    if (chip->log_mode)
        chip->setlogdata(spe->unitid, inst->pc, MEM_INST_READ);
}

/**********************************************************************/
inline void Spu::decode(SpuInst *inst)
{
    inst->decode();
}

/**********************************************************************/
inline void Spu::execute(SpuInst *inst) 
{
    if (inst->pipe == PIPE_EVEN)
        executeeven(inst);
    else
        executeodd(inst);
}

/**********************************************************************/
inline void Spu::load(uint_032t addr, Register * rt)
{
    uint_032t data;
    addr = addr & as->lslr & 0xfffffff0;
    for (uint i = 0; i < NSLOT(uint_032t); i++) {
        spe->ls->read4b(addr + (i * sizeof(uint_032t)), &data);
        rt->put032t(i, data);
    }
    if (chip->log_mode)
        chip->setlogdata(spe->unitid, addr, MEM_READ);
}

/**********************************************************************/
inline void Spu::store(uint_032t addr, Register * rt)
{
    addr = addr & as->lslr & 0xfffffff0;
    for (uint i = 0; i < NSLOT(uint_032t); i++)
        spe->ls->write4b(addr + (i * sizeof(uint_032t)), rt->get032t(i));
    if (chip->log_mode)
        chip->setlogdata(spe->unitid, addr, MEM_WRITE);
}

/**********************************************************************/
inline void Spu::lsaccess(SpuInst *inst)
{
    if (inst->isload())
        load(inst->targ, inst->rrt);
    else if (inst->isstore())
        store(inst->targ, inst->rrt);
}

/**********************************************************************/ 
inline void Spu::writeback(SpuInst *inst) 
{
    if (inst->type & IT_W_RT)
        as->reg[inst->rt]->put128t(inst->rrt->get128t());
}

/**********************************************************************/ 
inline void Spu::setnextpc(SpuInst *inst)
{
    if(inst->nextstate != SPE_STALL)
        as->pc = inst->nextpc;
}

/**********************************************************************/ 
inline void Spu::executeodd(SpuInst *inst)
{
    Register *rra = inst->rra;
    Register *rrb = inst->rrb;
    Register *rrc = inst->rrc;
    Register *rrt = inst->rrt;
    uint imm = inst->imm;
    uint pc = inst->pc;
    uint_032t targ = inst->targ;
    uint_032t nextpc = inst->nextpc;
    
    if (chip->debug_mode == 2)
        inst->exprinthead();
    switch (inst->op) {
    case LQD______:
    case STQD_____:
        
        // determine load/store target address
        targ = (exts32(imm, 10) << 4) + rra->get032t(PSLOT);
        break;
    case LQX______:
    case STQX_____:
        targ = rra->get032t(PSLOT) + rrb->get032t(PSLOT);
        break;
    case LQA______:
    case STQA_____:
        targ = exts32(imm, 16) << 2;
        break;
    case LQR______:
    case STQR_____:
        targ = (exts32(imm, 16) << 2) + pc;
        break;
    case CBD______:
    {
        int t = (rra->get032t(PSLOT) + exts32(imm, 7)) & 0xf;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put008t(t, 0x03);
        break;
    }
    case CBX______:
    {
        int t = (rra->get032t(PSLOT) + rrb->get032t(PSLOT)) & 0xf;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put008t(t, 0x03);
        break;
    }
    case CHD______:
    {
        int t = (rra->get032t(PSLOT) + exts32(imm, 7)) & 0xe;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put016t(t >> 1, 0x0203);
        break;
    }
    case CHX______:
    {
        int t = (rra->get032t(PSLOT) + rrb->get032t(PSLOT)) & 0xe;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put016t(t >> 1, 0x0203);
        break;
    }
    case CWD______:
    {
        int t = (rra->get032t(PSLOT) + exts32(imm, 7)) & 0xc;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put032t(t >> 2, 0x00010203);
        break;
    }
    case CWX______:
    {
        int t = (rra->get032t(PSLOT) + rrb->get032t(PSLOT)) & 0xc;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put032t(t >> 2, 0x00010203);
        break;
    }
    case CDD______:
    {
        int t = (rra->get032t(PSLOT) + exts32(imm, 7)) & 0x8;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put064t(t >> 3, 0x0001020304050607ull);
        break;
    }
    case CDX______:
    {
        int t = (rra->get032t(PSLOT) + rrb->get032t(PSLOT)) & 0x8;
        rrt->put064t(0, 0x1011121314151617ull);
        rrt->put064t(1, 0x18191a1b1c1d1e1full);
        rrt->put064t(t >> 3, 0x0001020304050607ull);
        break;
    }
    
    /********/ 
    case FSMBI____:
    {
        uint_032t s = imm;
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, (s & (0x8000 >> i)) ? 0xff : 0x00);
        break;
    }
    
    /********/ 
    case FSMB_____:
    {
        uint_032t s = rra->get032t(PSLOT);
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, (s & (0x8000 >> i)) ? 0xff : 0x00);
        break;
    }
    case FSMH_____:
    {
        uint_032t s = rra->get032t(PSLOT);
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (s & (0x80 >> i)) ? 0xffff : 0x0000);
        break;
    }
    case FSM______:
    {
        uint_032t s = rra->get032t(PSLOT);
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            rrt->put032t(i,
                         (s & (0x8 >> i)) ? 0xffffffff : 0x00000000);
        } break;
    }
    case GBB______:
    {
        uint_032t s = 0;
        for (int i = 0; i < NSLOT(uint_008t); i++)
            s = (s << 1) | (rra->get008t(i) & 0x1);
        rrt->put032t(0, s);
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        break;
    }
    case GBH______:
    {
        uint_032t s = 0;
        for (int i = 0; i < NSLOT(uint_016t); i++)
            s = (s << 1) | (rra->get016t(i) & 0x1);
        rrt->put032t(0, s);
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        break;
    }
    case GB_______:
    {
        uint_032t s = 0;
        for (int i = 0; i < NSLOT(uint_032t); i++)
            s = (s << 1) | (rra->get032t(i) & 0x1);
        rrt->put032t(0, s);
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        break;
    }
    case ORX______:
        rrt->put032t(0,
                     rra->get032t(0) | rra->get032t(1) | rra->
                     get032t(2) | rra->get032t(3));
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        break;
    case SHUFB____:
    {
        uint_008t c;
        for (int i = 0; i < NSLOT(uint_008t); i++) {
            uint_008t b = rrc->get008t(i);
            if ((b & 0xc0) == 0x80)    // 10xxxxxx
                c = 0x00;
            
            else if ((b & 0xe0) == 0xc0)    // 110xxxxx
                c = 0xff;
            
            else if ((b & 0xe0) == 0xe0)    // 111xxxxx
                c = 0x80;
            
            else if (b & 0x10)  // 0xx1xxxx
                c = rrb->get008t(b & 0xf);
            
            else            // 0xx0xxxx
                c = rra->get008t(b & 0xf);
            rrt->put008t(i, c);
        }
        break;
    }
    
    /********/ 
    case SHLQBI___:
    {
        int s = rrb->get032t(PSLOT) & 0x7;
        if (s != 0) {
            rrt->put064t(0, (rra->get064t(0) << s) | 
                         (rra->get064t(1) >> (64 - s)));
            rrt->put064t(1, (rra->get064t(1) << s));
        } else {
            rrt->put128t(rra->get128t());
        }
        break;
    }
    case SHLQBII__:
    {
        int s = imm & 0x7;
        if (s != 0) {
            rrt->put064t(0, (rra->get064t(0) << s) | 
                         (rra->get064t(1) >> (64 - s)));
            rrt->put064t(1, (rra->get064t(1) << s));
        } else {
            rrt->put128t(rra->get128t());
        }
        break;
    }
    case SHLQBY___:
    {
        int s = rrb->get032t(PSLOT) & 0x1f;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, (b + s < 16) ? rra->get008t(b + s) : 0);
        break;
    }
    case SHLQBYI__:
    {
        int s = imm & 0x1f;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, (b + s < 16) ? rra->get008t(b + s) : 0);
        break;
    }
    case SHLQBYBI_:
    {
        int s = (rrb->get032t(PSLOT) >> 3) & 0x1f;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, (b + s < 16) ? rra->get008t(b + s) : 0);
        break;
    }
    case ROTQBY___:
    {
        int s = rrb->get032t(PSLOT) & 0xf;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, rra->get008t((b + s) & 0xf));
        break;
    }
    case ROTQBYI__:
    {
        int s = imm & 0xf;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, rra->get008t((b + s) & 0xf));
        break;
    }
    case ROTQBYBI_:
    {
        int s = (rrb->get032t(PSLOT) >> 3) & 0xf;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, rra->get008t((b + s) & 0xf));
        break;
    }
    case ROTQBI___:
    {
        int s = rrb->get032t(PSLOT) & 0x7;
        if (s != 0) {
            rrt->put064t(0, (rra->get064t(0) << s) | 
                         (rra->get064t(1) >> (64 - s)));
            rrt->put064t(1, (rra->get064t(1) << s) | 
                         (rra->get064t(0) >> (64 - s)));
        } else {
            rrt->put128t(rra->get128t());
        }
        break;
    }
    case ROTQBII__:
    {
        int s = imm & 0x7;
        if (s != 0) {
            rrt->put064t(0, (rra->get064t(0) << s) | 
                         (rra->get064t(1) >> (64 - s)));
            rrt->put064t(1, (rra->get064t(1) << s) | 
                         (rra->get064t(0) >> (64 - s)));
        } else {
            rrt->put128t(rra->get128t());
        }
        break;
    }
    case ROTQMBY__:
    {
        int s = (0 - rrb->get032t(PSLOT)) & 0x1f;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, (b >= s) ? rra->get008t(b - s) : 0x00);
        break;
    }
    case ROTQMBYI_:
    {
        int s = (0 - imm) & 0x1f;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, (b >= s) ? rra->get008t(b - s) : 0x00);
        break;
    }
    case ROTQMBYBI:
    {
        int s = (0 - (rrb->get032t(PSLOT) >> 3)) & 0x1f;
        for (int b = 0; b < NSLOT(uint_008t); b++)
            rrt->put008t(b, (b >= s) ? rra->get008t(b - s) : 0x00);
        break;
    }
    case ROTQMBI__:
    {
        int s = (0 - rrb->get032t(PSLOT)) & 0x7;
        if (s != 0) {
            rrt->put064t(0, (rra->get064t(0) >> s));
            rrt->put064t(1, (rra->get064t(1) >> s) | 
                         (rra->get064t(0) << (64 - s)));
        } else {
            rrt->put128t(rra->get128t());
        }
        break;
    }
    case ROTQMBII_:
    {
        int s = (0 - imm) & 0x7;
        if (s != 0) {
            rrt->put064t(0, (rra->get064t(0) >> s));
            rrt->put064t(1, (rra->get064t(1) >> s) | 
                         (rra->get064t(0) << (64 - s)));
        } else {
            rrt->put128t(rra->get128t());
        }
        break;
    }
    
    /********/ 
    case BR_______:
        
        // determine branch target address
        nextpc = (pc + exts32(imm << 2, 18)) & as->lslr;
        break;
    case BRA______:
        nextpc = (exts32(imm << 2, 18)) & as->lslr;
        break;
    case BRSL_____:
        rrt->put032t(PSLOT, (pc + 4) & as->lslr);
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        nextpc = (pc + exts32(imm << 2, 18)) & as->lslr;
        break;
    case BRASL____:
        rrt->put032t(PSLOT, (pc + 4) & as->lslr);
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        nextpc = (exts32(imm << 2, 18)) & as->lslr;
        break;
    case BI_______:
        nextpc = rra->get032t(PSLOT) & as->lslr & 0xfffffffc;
        
        // enable/disable interrupts by E/D function bits //
        break;
    case BISL_____:
        rrt->put032t(PSLOT, (pc + 4) & as->lslr);
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        nextpc = rra->get032t(PSLOT) & as->lslr & 0xfffffffc;
        
        // enable/disable interrupts by E/D function bits //
        break;
    case BRNZ_____:
        if (rrt->get032t(PSLOT) != 0)
            nextpc = (pc + exts32(imm << 2, 18)) & as->lslr;
        break;
    case BRZ______:
        if (rrt->get032t(PSLOT) == 0)
            nextpc = (pc + exts32(imm << 2, 18)) & as->lslr;
        break;
    case BRHNZ____:
        if ((rrt->get032t(PSLOT) & 0x0000ffff) != 0)
            nextpc = (pc + exts32(imm << 2, 18)) & as->lslr;
        break;
    case BRHZ_____:
        if ((rrt->get032t(PSLOT) & 0x0000ffff) == 0)
            nextpc = (pc + exts32(imm << 2, 18)) & as->lslr;
        break;
    case BIZ______:
        if (rrt->get032t(PSLOT) == 0) {
            nextpc = rra->get032t(PSLOT) & as->lslr & 0xfffffffc;
            
            // enable/disable interrupts by E/D function bits //
        }
        break;
    case BINZ_____:
        if (rrt->get032t(PSLOT) != 0) {
            nextpc = rra->get032t(PSLOT) & as->lslr & 0xfffffffc;
            
            // enable/disable interrupts by E/D function bits //
        }
        break;
    case BIHZ_____:
        if ((rrt->get032t(PSLOT) & 0x0000ffff) == 0) {
            nextpc = rra->get032t(PSLOT) & as->lslr & 0xfffffffc;
            
            // enable/disable interrupts by E/D function bits //
        }
        break;
    case BIHNZ____:
        if ((rrt->get032t(PSLOT) & 0x0000ffff) != 0) {
            nextpc = rra->get032t(PSLOT) & as->lslr & 0xfffffffc;
            
            // enable/disable interrupts by E/D function bits //
        }
        break;
        
        /********/ 
    case HBR______:
        
        // determine branch target address
        // hinted branch address has been computed and written in 'ro'
        targ = rra->get032t(PSLOT) & as->lslr & 0xfffffffc;
        break;
    case HBRA_____:
        targ = exts32(imm << 2, 18) & as->lslr;
        break;
    case HBRR_____:
        targ = (exts32(imm << 2, 18) + pc) & as->lslr;
        break;
        
        /********/ 
    case FREST____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i, 1 / rra->getf32t(i));
        break;
    case FRSQEST__:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i, 1 / sqrt(fabs(rra->getf32t(i))));
        break;
        
        /********/ 
    case STOP_____:
        inst->nextstate = SPE_STOP;
        as->stop_reason = SR_STOP | (inst->ir & SR_SIGNALMASK);
        break;
    case STOPD____:
        inst->nextstate = SPE_STOP;
        as->stop_reason = SR_STOP | (inst->ir & SR_SIGNALMASK);
        break;
    case LNOP_____:
    case SYNC_____:
    case DSYNC____:
        break;
    case MFSPR____:
        rrt->put064t(0, 0);
        rrt->put064t(1, 0);
        break;
    case MTSPR____:
        break;
        
        /********/ 
    case RDCH_____:
    {
        uint_032t data;
        if (spe->chi->spureadch(inst->ca, &data)) {
            inst->nextstate = SPE_STALL;
            ss->stall_reason = ST_NOTRDCH + inst->ca;
            break;
        }
        rrt->put032t(PSLOT, data);
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        break;
    }
    case WRCH_____:
        if (spe->chi->spuwritech(inst->ca, rrt->get032t(PSLOT))) {
            inst->nextstate = SPE_STALL;
            ss->stall_reason = ST_NOTWRCH + inst->ca;
        }
        break;
    case RCHCNT___:
        rrt->put032t(PSLOT, spe->chi->spureadchcnt(inst->ca));
        rrt->put032t(1, 0);
        rrt->put032t(2, 0);
        rrt->put032t(3, 0);
        break;
    default:
        printf("!! %5x:", (uint) pc);
        for (int i = 0; i < 4; i++)
            printf(" %02x", (uint) ((inst->ir >> (24 - i * 8)) & 0xff));
        printf("\n%s is not supported yet. Stop.\n", inst->getopname());
        inst->nextstate = SPE_STOP;
        as->stop_reason = SR_ERROR;
        break;
    }
    if (chip->debug_mode == 2)
        inst->exprinttail();
    inst->targ = targ;
    inst->nextpc = nextpc;
}

/**********************************************************************/ 
inline void Spu::executeeven(SpuInst *inst) 
{
    Register *rra = inst->rra;
    Register *rrb = inst->rrb;
    Register *rrc = inst->rrc;
    Register *rrt = inst->rrt;
    uint imm = inst->imm;
    
    if (chip->debug_mode == 2)
        inst->exprinthead();
    switch (inst->op) {
    case ILH______:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, imm);
        break;
    case ILHU_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, imm << 16);
        break;
    case IL_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, exts32(imm, 16));
        break;
    case ILA______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, imm);
        break;
    case IOHL_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rrt->get032t(i) | imm);
        break;
        
        /********/ 
    case AH_______:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, rra->get016t(i) + rrb->get016t(i));
        break;
    case AHI______:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, rra->get016t(i) + exts32(imm, 10));
        break;
    case A________:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) + rrb->get032t(i));
        break;
    case AI_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) + exts32(imm, 10));
        break;
    case SFH______:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, rrb->get016t(i) - rra->get016t(i));
        break;
    case SFHI_____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, exts32(imm, 10) - rra->get016t(i));
        break;
    case SF_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rrb->get032t(i) - rra->get032t(i));
        break;
    case SFI______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, exts32(imm, 10) - rra->get032t(i));
        break;
    case ADDX_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         rra->get032t(i) + rrb->get032t(i) +
                         (rrt->get032t(i) & 0x1));
        break;
    case CG_______:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_032t data = rra->get032t(i) + rrb->get032t(i);
            data = (data < rra->get032t(i));
            rrt->put032t(i, data);
        } break;
    }
    case CGX______:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_032t data = rra->get032t(i) + rrb->get032t(i);
            data = ((data < rra->get032t(i)) || 
                    ((data == 0xffffffff)
                     && (rrt->get032t(i) & 0x1)));
            rrt->put032t(i, data);
        } break;
    }
    case SFX______:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_032t data = rrb->get032t(i) - rra->get032t(i);
            data += (rrt->get032t(i) & 0x1) - 1;
            rrt->put032t(i, data);
        } break;
    }
    case BG_______:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_032t data = (rrb->get032t(i) >= rra->get032t(i));
            rrt->put032t(i, data);
        } break;
    }
    case BGX______:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_032t data = (rrt->get032t(i) & 0x1) ? 
                (rrb->get032t(i) >= rra->get032t(i)) : 
                (rrb->get032t(i) > rra->get032t(i));
            rrt->put032t(i, data);
        } break;
    }
    case MPY______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         low16(rra->get032t(i)) * low16(rrb->get032t(i)));
        break;
    case MPYU_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         ulow16(rra->get032t(i)) *
                         ulow16(rrb->get032t(i)));
        break;
    case MPYI_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, low16(rra->get032t(i)) * exts32(imm, 10));
        break;
    case MPYUI____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         ulow16(rra->get032t(i)) *
                         ulow16(exts32(imm, 10)));
        break;
    case MPYA_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         (low16(rra->get032t(i)) *
                          low16(rrb->get032t(i)) + rrc->get032t(i)));
        break;
    case MPYH_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         (high16(rra->get032t(i)) *
                          ulow16(rrb->get032t(i))) << 16);
        break;
    case MPYS_____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_032t data =
                low16(rra->get032t(i)) * low16(rrb->get032t(i));
            rrt->put032t(i, exts32(data >> 16, 16));
        } break;
    }
    case MPYHH____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         high16(rra->get032t(i)) *
                         high16(rrb->get032t(i)));
        break;
    case MPYHHA___:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         high16(rra->get032t(i)) *
                         high16(rrb->get032t(i)) + rrt->get032t(i));
        break;
    case MPYHHU___:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         (uhigh16(rra->get032t(i)) *
                          uhigh16(rrb->get032t(i))));
        break;
    case MPYHHAU__:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         (uhigh16(rra->get032t(i)) *
                          uhigh16(rrb->get032t(i)) + rrt->get032t(i)));
        break;
    case CLZ______:
    {
        uint_032t t;
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_032t u = rra->get032t(i);
            for (t = 0; t < 32; t++)
                if (u & (0x80000000 >> t))
                    break;
            rrt->put032t(i, t);
        }
        break;
    }
    case CNTB_____:
    {
        for (int i = 0; i < NSLOT(uint_008t); i++) {
            uint_008t b = rra->get008t(i);
            uint_008t c = 0;
            for (uint m = 0; m < 8; m++) {
                c += ((b & (0x80 >> m)) != 0);
            }
            rrt->put008t(i, c);
        }
        break;
    }
    case AVGB_____:
    {
        for (int i = 0; i < NSLOT(uint_008t); i++) {
            uint_016t data =
                (rra->get008t(i) + rrb->get008t(i) + 1) >> 1;
            rrt->put008t(i, (uint_008t) data);
        } break;
    }
    case ABSDB____:
    {
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i,
                         ((rrb->get008t(i) >
                           rra->get008t(i)) ? rrb->get008t(i) -
                          rra->get008t(i) : rra->get008t(i) -
                          rrb->get008t(i)));
        break;
    }
    case SUMB_____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            uint_016t ra_sum = 0, rb_sum = 0;
            for (uint j = 0; j < 4; j++) {
                ra_sum += rrb->get008t(i * 4 + j);
                rb_sum += rra->get008t(i * 4 + j);
            }
            rrt->put016t(i * 2, ra_sum);
            rrt->put016t(i * 2 + 1, rb_sum);
        }
        break;
    }
    case XSBH_____:
    {
        for (int i = 0; i < NSLOT(uint_016t); i++) {
            uint_016t data =
                ulow16(exts32(rra->get008t(i * 2 + 1), 8));
            rrt->put016t(i, data);
        } break;
    }
    case XSHW_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, exts32(rra->get032t(i), 16));
        break;
    case XSWD_____:
    {
        for (int i = 0; i < NSLOT(uint_064t); i++) {
            uint_064t data = rra->get032t(i * 2 + 1);
            if (data & 0x80000000)
                data = data | 0xffffffff00000000ull;
            rrt->put064t(i, data);
        }
        break;
    }
    case AND______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) & rrb->get032t(i));
        break;
    case ANDC_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) & ~rrb->get032t(i));
        break;
    case ANDBI____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, rra->get008t(i) & imm);
        break;
    case ANDHI____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, rra->get016t(i) & exts32(imm, 10));
        break;
    case ANDI_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) & exts32(imm, 10));
        break;
    case OR_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) | rrb->get032t(i));
        break;
    case ORC______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) | ~rrb->get032t(i));
        break;
    case ORBI_____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, rra->get008t(i) | imm);
        break;
    case ORHI_____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, rra->get016t(i) | exts32(imm, 10));
        break;
    case ORI______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) | exts32(imm, 10));
        break;
    case XOR______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) ^ rrb->get032t(i));
        break;
    case XORBI____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, rra->get008t(i) ^ imm);
        break;
    case XORHI____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, rra->get016t(i) ^ exts32(imm, 10));
        break;
    case XORI_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) ^ exts32(imm, 10));
        break;
    case NAND_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, ~(rra->get032t(i) & rrb->get032t(i)));
        break;
    case NOR______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, ~(rra->get032t(i) | rrb->get032t(i)));
        break;
    case EQV______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, rra->get032t(i) ^ ~rrb->get032t(i));
        break;
    case SELB_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, ((rrc->get032t(i) & rrb->get032t(i)) | 
                             (~rrc->get032t(i) & rra->get032t(i))));
        break;
        
        /********/ 
    case SHLH_____:
    {
        for (int i = 0; i < NSLOT(uint_016t); i++) {
            int s = rrb->get016t(i) & 0x1f;
            rrt->put016t(i, (s < 16) ? rra->get016t(i) << s : 0);
        } break;
    }
    case SHLHI____:
    {
        int s = imm & 0x1f;
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (s < 16) ? rra->get016t(i) << s : 0);
        break;
    }
    case SHL______:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            int s = rrb->get032t(i) & 0x3f;
            rrt->put032t(i, (s < 32) ? rra->get032t(i) << s : 0);
        } break;
    }
    case SHLI_____:
    {
        int s = imm & 0x3f;
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (s < 32) ? rra->get032t(i) << s : 0);
        break;
    }
    case ROTH_____:
    {
        for (int i = 0; i < NSLOT(uint_016t); i++) {
            int s = rrb->get016t(i) & 0xf;
            rrt->put016t(i, ((rra->get016t(i) << s) | 
                             (rra->get016t(i) >> (16 - s))));
        } break;
    }
    case ROTHI____:
    {
        int s = imm & 0xf;
        for (int i = 0; i < NSLOT(uint_016t); i++) {
            rrt->put016t(i, ((rra->get016t(i) << s) | 
                             (rra->get016t(i) >> (16 - s))));
        } break;
    }
    case ROT______:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            int s = rrb->get032t(i) & 0x1f;
            rrt->put032t(i, ((rra->get032t(i) << s) | 
                             (rra->get032t(i) >> (32 - s))));
        } break;
    }
    case ROTI_____:
    {
        int s = imm & 0x1f;
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            rrt->put032t(i, ((rra->get032t(i) << s) | 
                             (rra->get032t(i) >> (32 - s))));
        } break;
    }
    case ROTHM____:
    {
        for (int i = 0; i < NSLOT(uint_016t); i++) {
            int s = (0 - rrb->get016t(i)) & 0x1f;
            rrt->put016t(i, (s < 16) ? rra->get016t(i) >> s : 0);
        } break;
    }
    case ROTHMI___:
    {
        int s = (0 - exts32(imm, 7)) & 0x1f;
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (s < 16) ? rra->get016t(i) >> s : 0);
        break;
    }
    case ROTM_____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            int s = (0 - rrb->get032t(i)) & 0x3f;
            rrt->put032t(i, (s < 32) ? rra->get032t(i) >> s : 0);
        } break;
    }
    case ROTMI____:
    {
        int s = (0 - exts32(imm, 7)) & 0x3f;
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (s < 32) ? rra->get032t(i) >> s : 0);
        break;
    }
    case ROTMAH___:
    {
        for (int i = 0; i < NSLOT(uint_016t); i++) {
            int s = (0 - rrb->get016t(i)) & 0x1f;
            s = (s < 16) ? s : 15;
            rrt->put016t(i, (uint_016t) exts32(rra->get016t(i) >> s, 
                                               (16 - s)));
        } break;
    }
    case ROTMAHI__:
    {
        int s = (0 - exts32(imm, 7)) & 0x1f;
        s = (s < 16) ? s : 15;
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (uint_016t) exts32(rra->get016t(i) >> s, 
                                               (16 - s)));
        break;
    }
    case ROTMA____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            int s = (0 - rrb->get032t(i)) & 0x3f;
            s = (s < 32) ? s : 31;
            rrt->put032t(i, exts32(rra->get032t(i) >> s, (32 - s)));
        } break;
    }
    case ROTMAI___:
    {
        int s = (0 - exts32(imm, 7)) & 0x3f;
        s = (s < 32) ? s : 31;
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, exts32(rra->get032t(i) >> s, (32 - s)));
        break;
    }
    
    /********/ 
    case HEQ______:
        if (rra->get032t(PSLOT) == rrb->get032t(PSLOT)) {
            inst->nextstate = SPE_STOP;
            as->stop_reason = SR_HALT;
        }
        break;
    case HEQI_____:
        if (rra->get032t(PSLOT) == exts32(imm, 10)) {
            inst->nextstate = SPE_STOP;
            as->stop_reason = SR_HALT;
        }
        break;
    case HGT______:
        if ((int) rra->get032t(PSLOT) > (int) rrb->get032t(PSLOT)) {
            inst->nextstate = SPE_STOP;
            as->stop_reason = SR_HALT;
        }
        break;
    case HGTI_____:
        if ((int) rra->get032t(PSLOT) > (int) exts32(imm, 10)) {
            inst->nextstate = SPE_STOP;
            as->stop_reason = SR_HALT;
        }
        break;
    case HLGT_____:
        if (rra->get032t(PSLOT) > rrb->get032t(PSLOT)) {
            inst->nextstate = SPE_STOP;
            as->stop_reason = SR_HALT;
        }
        break;
    case HLGTI____:
        if (rra->get032t(PSLOT) > exts32(imm, 10)) {
            inst->nextstate = SPE_STOP;
            as->stop_reason = SR_HALT;
        }
        break;
    case CEQB_____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, (rra->get008t(i) == rrb->get008t(i)) 
                         ? 0xff : 0x00);
        break;
    case CEQBI____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, (rra->get008t(i) == (imm & 0xff)) 
                         ? 0xff : 0x00);
        break;
    case CEQH_____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (rra->get016t(i) == rrb->get016t(i)) 
                         ? 0xffff : 0x0000);
        break;
    case CEQHI____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (rra->get016t(i) == ulow16(exts32(imm, 10))) 
                         ? 0xffff : 0x0000);
        break;
    case CEQ______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (rra->get032t(i) == rrb->get032t(i)) 
                         ? 0xffffffff : 0x00000000);
        break;
    case CEQI_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (rra->get032t(i) == exts32(imm, 10)) 
                         ? 0xffffffff : 0x00000000);
        break;
    case CGTB_____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, ((sint_008t) rra->get008t(i) > 
                             (sint_008t) rrb->get008t(i)) 
                         ? 0xff : 0x00);
        break;
    case CGTBI____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, ((sint_008t) rra->get008t(i) > 
                             (sint_008t) (imm & 0xff))  ? 0xff : 0x00);
        break;
    case CGTH_____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, ((sint_016t) rra->get016t(i) > 
                             (sint_016t) rrb->get016t(i)) 
                         ? 0xffff : 0x0000);
        break;
    case CGTHI____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, ((sint_016t) rra->get016t(i) > 
                             (sint_016t) exts32(imm, 10)) 
                         ? 0xffff : 0x0000);
        break;
    case CGT______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, ((sint_032t) rra->get032t(i) > 
                             (sint_032t) rrb->get032t(i)) 
                         ? 0xffffffff : 0x00000000);
        break;
    case CGTI_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, ((sint_032t) rra->get032t(i) > 
                             (sint_032t) exts32(imm, 10)) 
                         ? 0xffffffff : 0x00000000);
        break;
    case CLGTB____:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, (rra->get008t(i) > rrb->get008t(i)) 
                         ? 0xff : 0x00);
        break;
    case CLGTBI___:
        for (int i = 0; i < NSLOT(uint_008t); i++)
            rrt->put008t(i, (rra->get008t(i) > (imm & 0xff)) 
                         ? 0xff : 0x00);
        break;
    case CLGTH____:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (rra->get016t(i) > rrb->get016t(i)) 
                         ? 0xffff : 0x0000);
        break;
    case CLGTHI___:
        for (int i = 0; i < NSLOT(uint_016t); i++)
            rrt->put016t(i, (rra->get016t(i) > ulow16(exts32(imm, 10))) 
                         ? 0xffff : 0x0000);
        break;
    case CLGT_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (rra->get032t(i) > rrb->get032t(i)) 
                         ? 0xffffffff : 0x00000000);
        break;
    case CLGTI____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (rra->get032t(i) > exts32(imm, 10)) 
                         ? 0xffffffff : 0x00000000);
        break;
        
        /********/ 
    case FA_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i, rra->getf32t(i) + rrb->getf32t(i));
        break;
    case DFA______:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i, rra->getf64t(i) + rrb->getf64t(i));
        break;
    case FS_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i, rra->getf32t(i) - rrb->getf32t(i));
        break;
    case DFS______:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i, rra->getf64t(i) - rrb->getf64t(i));
        break;
    case FM_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i, rra->getf32t(i) * rrb->getf32t(i));
        break;
    case DFM______:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i, rra->getf64t(i) * rrb->getf64t(i));
        break;
    case FMA______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i,
                         (rra->getf32t(i) * rrb->getf32t(i) +
                          rrc->getf32t(i)));
        break;
    case DFMA_____:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i,
                         (rra->getf64t(i) * rrb->getf64t(i) +
                          rrt->getf64t(i)));
        break;
    case FNMS_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i,
                         (rrc->getf32t(i) -
                          rra->getf32t(i) * rrb->getf32t(i)));
        break;
    case DFNMS____:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i,
                         (rrt->getf64t(i) -
                          rra->getf64t(i) * rrb->getf64t(i)));
        break;
    case FMS______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i,
                         (rra->getf32t(i) * rrb->getf32t(i) -
                          rrc->getf32t(i)));
        break;
    case DFMS_____:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i,
                         (rra->getf64t(i) * rrb->getf64t(i) -
                          rrt->getf64t(i)));
        break;
    case DFNMA____:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i,
                         (-rra->getf64t(i) * rrb->getf64t(i) -
                          rrt->getf64t(i)));
        break;
    case FI_______:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->putf32t(i, rrb->getf32t(i));
        break;
    case CSFLT____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            float fdata = (float) ((sint_032t) rra->get032t(i));
            for (int j = 0; j < 155 - (sint_032t) imm; j++)
                fdata /= 2;
            rrt->putf32t(i, fdata);
        } break;
    }
    case CFLTS____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            float fdata = rra->getf32t(i);
            for (int j = 0; j < 173 - (sint_032t) imm; j++)
                fdata *= 2;
            if (fdata > 0x7fffffffll)
                rrt->put032t(i, 0x7fffffff);
            
            else if (fdata < -0x80000000ll)
                rrt->put032t(i, 0x80000000);
            
            else
                rrt->put032t(i, (uint_032t) ((sint_032t) fdata));
        }
        break;
    }
    case CUFLT____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            float fdata = (float) rra->get032t(i);
            for (int j = 0; j < 155 - (sint_032t) imm; j++)
                fdata /= 2;
            rrt->putf32t(i, fdata);
        } break;
    }
    case CFLTU____:
    {
        for (int i = 0; i < NSLOT(uint_032t); i++) {
            float fdata = rra->getf32t(i);
            for (int j = 0; j < 173 - (sint_032t) imm; j++)
                fdata *= 2;
            if (fdata > 0xffffffffll)
                rrt->put032t(i, 0xffffffff);
            
            else if (fdata < 0)
                rrt->put032t(i, 0);
            
            else
                rrt->put032t(i, (uint_032t) fdata);
        }
        break;
    }
    case FRDS_____:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf32t(i * 2, (float) rra->getf64t(i));
        break;
    case FESD_____:
        for (int i = 0; i < NSLOT(uint_064t); i++)
            rrt->putf64t(i, (double) rra->getf32t(i * 2));
        break;
    case FCEQ_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (rra->getf32t(i) == rrb->getf32t(i)) 
                         ? 0xffffffff : 0x00000000);
        break;
    case FCMEQ____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         (fabs(rra->getf32t(i)) ==
                          fabs(rrb->
                               getf32t(i)))  ? 0xffffffff : 0x00000000);
        break;
    case FCGT_____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i, (rra->getf32t(i) > rrb->getf32t(i)) 
                         ? 0xffffffff : 0x00000000);
        break;
    case FCMGT____:
        for (int i = 0; i < NSLOT(uint_032t); i++)
            rrt->put032t(i,
                         (fabs(rra->getf32t(i)) >
                          fabs(rrb->
                               getf32t(i)))  ? 0xffffffff : 0x00000000);
        break;
    case FSCRWR___:
        as->fpscr->write(rra->get128t());
        break;
    case FSCRRD___:
        rrt->put128t(as->fpscr->read());
        break;
        
        /********/ 
    case NOP______:
        break;
    default:
        printf("!! %5x:", (uint) inst->pc);
        for (int i = 0; i < 4; i++)
            printf(" %02x", (uint) ((inst->ir >> (24 - i * 8)) & 0xff));
        printf("\n%s is not supported yet. Stop.\n", inst->getopname());
        inst->nextstate = SPE_STOP;
        as->stop_reason = SR_ERROR;
        break;
    }
    if (chip->debug_mode == 2)
        inst->exprinttail();
}

/**********************************************************************/ 
LocalStore::LocalStore(ullint size)
{
    int n = size / sizeof(uint_032t);
    mem = new uint_032t[n];
}

/**********************************************************************/
LocalStore::~LocalStore()
{
    delete[] mem;
}

/**********************************************************************/
void LocalStore::read1b(uint_032t addr, uint_008t * data)
{
    *data = (mem[addr / sizeof(uint_032t)] >> 
             (24 - (addr % sizeof(uint_032t)) * 8)) & 0xff;
}

/**********************************************************************/
void LocalStore::read4b(uint_032t addr, uint_032t *data)
{
    *data = mem[addr / sizeof(uint_032t)];
}

/**********************************************************************/
void LocalStore::readnb(uint_032t addr, int size, uint_032t data[])
{
    int i, j, index;
    uint_032t temp = 0;
    uint_008t c;
    
    index = 0;
    for (i = 0; i < size; i += sizeof(uint_032t)) {
        read4b(addr + i, &data[index]);
        index++;
    }
    for (j = 0; i + j < size; j++) {
        read1b(addr + i + j, &c);
        temp = (temp << 8) || c;
    }
    if (j != 0)
        data[index] = temp << (32 - j * 8);
}

/**********************************************************************/
void LocalStore::write1b(uint_032t addr, uint_008t data)
{
    int offset = (24 - (addr % sizeof(uint_032t)) * 8);
    uint_032t ins = (uint_032t) data << offset;
    uint_032t mask = 0xff << offset;
    mem[addr / sizeof(uint_032t)] = 
        ins | (mem[addr / sizeof(uint_032t)] & ~mask);
}

/**********************************************************************/
void LocalStore::write4b(uint_032t addr, uint_032t data)
{
    mem[addr / sizeof(uint_032t)] = data;
}

/**********************************************************************/
void LocalStore::writenb(uint_032t addr, int size, uint_032t data[])
{
    int i, j, index;
    
    index = 0;
    for (i = 0; i < size; i += sizeof(uint_032t)) {
        write4b(addr + i, data[index]);
        index++;
    }
    for (j = 0; i + j < size; j++)
        write1b(addr + i + j, (data[index] >> (24 - j * 8)) & 0xff);
}

/**********************************************************************/
void LocalStore::print()
{
    uint_032t data;
    uint_032t check;
    uint_032t off;
    uint_032t nulstart = 0;
    char buf[256];
    char temp[10];
    printf("[LOCAL STORE(SIZE: %d)]\n", (int)LS_SIZE);
    for (off = 0; off < LS_SIZE; off += 16 * sizeof(uint_032t)) {
        buf[0] = '\0';
        check = 0;
        for (uint i = 0; i < 16; i++) {
            read4b(off + i * sizeof(uint_032t), &data);
            sprintf(temp, "%08x ", (uint) data);
            strcat(buf, temp);
            check = check | data;
        }
        if (check == 0) {
            if (nulstart == 0)
                nulstart = off;
        } else {
            if (nulstart != 0) {
                printf("\n%8x..%8x: null\n\n", (uint) nulstart,
                       (uint) off - 1);
                nulstart = 0;
            }
            printf("%8x: %s\n", (uint) off, buf);
        }
    }
    if (nulstart != 0)
        printf("\n%8x..%8x: null\n\n", (uint) nulstart,
               (uint)(off - sizeof(uint_032t)));
    return;
}

/**********************************************************************/
Register::Register(Chip * chip)
{
    this->chip = chip;
    for (uint i = 0; i < NSLOT(uint_064t); i++)
        reg.i064t[i] = 0;
}

/**********************************************************************/
void Register::print()
{
    printf("0x");
    for (uint j = 0; j < NSLOT(uint_032t); j++)
        printf("%08x", (uint) get032t(j));
    printf("\n");
}

/**********************************************************************/
uint_008t Register::get008t(int offset)
{
    return reg.i008t[slot(offset, NSLOT(uint_008t))];
}

/**********************************************************************/
uint_016t Register::get016t(int offset)
{
    return reg.i016t[slot(offset, NSLOT(uint_016t))];
}

/**********************************************************************/
uint_032t Register::get032t(int offset)
{
    return reg.i032t[slot(offset, NSLOT(uint_032t))];
}

/**********************************************************************/
uint_064t Register::get064t(int offset)
{
    return reg.i064t[slot(offset, NSLOT(uint_064t))];
}

/**********************************************************************/
uint_128t Register::get128t()
{
    return *(uint_128t *) & reg;
}

/**********************************************************************/
flot_032t Register::getf32t(int offset)
{
    return reg.f032t[slot(offset, NSLOT(uint_032t))];
}

/**********************************************************************/
flot_064t Register::getf64t(int offset)
{
    return reg.f064t[slot(offset, NSLOT(uint_064t))];
}

/**********************************************************************/
void Register::put008t(int offset, uint_008t data)
{
    reg.i008t[slot(offset, NSLOT(uint_008t))] = data;
}

/**********************************************************************/
void Register::put016t(int offset, uint_016t data)
{
    reg.i016t[slot(offset, NSLOT(uint_016t))] = data;
}

/**********************************************************************/
void Register::put032t(int offset, uint_032t data)
{
    reg.i032t[slot(offset, NSLOT(uint_032t))] = data;
}

/**********************************************************************/
void Register::put064t(int offset, uint_064t data)
{
    reg.i064t[slot(offset, NSLOT(uint_064t))] = data;
}

/**********************************************************************/
void Register::put128t(uint_128t data)
{
    reg = *(regdata_t *) & data;
}

/**********************************************************************/
void Register::putf32t(int offset, flot_032t data)
{
    reg.f032t[slot(offset, NSLOT(uint_032t))] = data;
}

/**********************************************************************/
void Register::putf64t(int offset, flot_064t data)
{
    reg.f064t[slot(offset, NSLOT(uint_064t))] = data;
}

/**********************************************************************/
Fpscr::Fpscr(Chip *)
{
    this->chip = chip;
    // enable = 00000f07 00003f07 00003f07 00000f07
    for (uint i = 0; i < NSLOT(uint_064t); i++) {
        reg.i064t[i] = 0;
        enable.i032t[i * 3] = 0x00000f07;
        enable.i032t[i + 1] = 0x00003f07;
    }
}

/**********************************************************************/
uint_128t Fpscr::read()
{
    regdata_t data;
    for (uint i = 0; i < NSLOT(uint_064t); i++)
        data.i064t[i] = reg.i064t[i] & enable.i064t[i];
    return *(uint_128t *) & data;
}

/**********************************************************************/
void Fpscr::write(uint_128t data)
{
    regdata_t temp = *(regdata_t *) & data;
    for (uint i = 0; i < NSLOT(uint_064t); i++)
        reg.i064t[i] = temp.i064t[i];
}

/**********************************************************************/
void Fpscr::bitset(int bit)
{
    uint_064t temp = 1 << (63 - (bit % 64));
    int index = slot(bit / 64, NSLOT(uint_064t));
    reg.i064t[index] = reg.i064t[index] | temp;
}

/**********************************************************************/
void Fpscr::twobitwrite(int bit, int data)
{
    uint_064t temp = (data & 0x3) << (62 - (bit % 64));
    uint_064t mask = 0x3 << (62 - (bit % 64));
    int index = slot(bit / 64, NSLOT(uint_064t));
    reg.i064t[index] = temp | (~mask & reg.i064t[index]);
}

/**********************************************************************/
SpeArchstate::SpeArchstate(Chip * chip)
{
    reg = new Register *[NREG];
    for (int i = 0; i < NREG; i++)
        reg[i] = new Register(chip);
    fpscr = new Fpscr(chip);
    
    this->chip = chip;
    pc = 0;
    lslr = LS_SIZE - 1;
    state = SPE_RUN;
}

/**********************************************************************/
SpeArchstate::~SpeArchstate()
{
    pc = 0;
    delete[]reg;
    reg = NULL;
    state = SPE_STOP;
}

/**********************************************************************/
SpeSimstate::SpeSimstate()
{
    instcnt = 0;
    stall_reason = 0;
}

/**********************************************************************/
Spe::Spe(Chip * chip, int id)
{
    this->chip = chip;
    chip->spe[id - 1] = this;
    ls = new LocalStore(LS_SIZE);
    chi = new ChannelInterface();
    as = new SpeArchstate(chip);
    ss = new SpeSimstate();
    spu = new Spu(chip, this);
    mfc = new Mfc(chip, this);
    this->unitid = id;
}

/**********************************************************************/
Spe::~Spe()
{
    delete ls;
    ls = NULL;
    delete chi;
    chi = NULL;
    delete as;
    as = NULL;
    delete ss;
    ss = NULL;
    delete spu;
    spu = NULL;
    delete mfc;
    mfc = NULL;
}

/** printf imprementation *********************************************/
/**********************************************************************/
enum {
    SPEFUNC_PUTCHAR = 24,
    SPEFUNC_PUTS = 25,
    SPEFUNC_PRINTF = 37,

    AL_032 = 4,
    AL_064 = 8,
    AL_F64 = 80,
    AL_NON = 0,
    AL_STR = -1,
    DUMMY = 1,
};

/**********************************************************************/
char *spegetstring(Spe * spe, uint_032t addr) 
{
    int length = 0;
    uint_008t c;
    char *result;
    
    do {
        spe->ls->read1b((addr + length) & spe->as->lslr, &c);
        length++;
    } while (c != '\0');
    result = new char[length];
    for (int i = 0; i < length; i++) {
        spe->ls->read1b((addr + i) & spe->as->lslr, &c);
        result[i] = (char) c;
    } return result;
}

/**********************************************************************/ 
inline void skipchar(char *targ, char *fmt, int *index) 
{
    while (strchr(targ, (int) *(fmt + *index)))
        (*index)++;
} 

/**********************************************************************/ 
int spevprintf(Spe * spe, char *fmt, uint_032t addr) 
{
    int total, index, head, arglen;
    char *temp = new char[strlen(fmt) + 1];
    total = 0;
    for (index = 0; *(fmt + index) != '\0'; index++) {
        if (*(fmt + index) == '%') {
            head = index;
            index++;
            strcpy(temp, fmt);
            skipchar("#0- +'I", fmt, &index);
            skipchar("0123456789*.", fmt, &index);
            if (strchr("cdipouxX", (int) *(fmt + index))) {
                arglen = AL_032;
            } else if (strchr("eEfFgG", (int) *(fmt + index))) {
                arglen = AL_F64;
            } else if (*(fmt + index) == 'h') {
                arglen = AL_032;
            } else if (*(fmt + index) == 'l') {
                index++;
                if (strchr("eEfFgG", (int) *(fmt + index))) {
                    arglen = AL_F64;
                } else if (*(fmt + index) == 'l') {
                    arglen = AL_064;
                    index++;
                } else {
                    arglen = AL_032;
                }
            } else if (*(fmt + index) == 's') {
                arglen = AL_STR;
            } else {
                arglen = AL_NON;
            }
            *(temp + index + 1) = '\0';
            if (arglen == AL_032) {
                uint_032t arg_032;
                spe->ls->read4b(addr, &arg_032);
                total += printf(temp + head, arg_032, DUMMY, DUMMY);
                addr += 16;
            } else if ((arglen == AL_064) || (arglen == AL_F64)) {
                united_064t arg_064;
                uint_032t high, low;
                spe->ls->read4b(addr, &high);
                spe->ls->read4b(addr + 4, &low);
                arg_064.lli = ((uint_064t) high << 32) | low;
                if (arglen == AL_064) {
                    total += printf(temp + head, arg_064.lli, DUMMY, DUMMY);
                } else {
                    float arg_flt = (float) arg_064.dbl;
                    total += printf(temp + head, arg_flt, DUMMY, DUMMY);
                }
                addr += 16;
            } else if (arglen == AL_STR) {
                uint_032t arg_addr;
                spe->ls->read4b(addr, &arg_addr);
                arg_addr = arg_addr & spe->as->lslr;
                char *arg_str = spegetstring(spe, arg_addr);
                total += printf(temp + head, arg_str, DUMMY, DUMMY);
                delete[]arg_str;
                addr += 16;
            } else {
                total += printf(temp + head, DUMMY, DUMMY);
            }
        } else {
            putchar((int) *(fmt + index));
            total++;
        }
    }
    delete[]temp;
    return total;
}

/**********************************************************************/ 
void spefunccall(Spe * spe) 
{
    uint_032t op, func, addr, arg, ret;
    spe->ls->read4b(spe->as->pc, &op);
    spe->as->pc += 4;
    func = op >> 24;
    addr = op & spe->as->lslr;
    spe->ls->read4b(addr, &arg);
    if (func == SPEFUNC_PUTCHAR) {
            // putchar()
        ret = putchar((int) arg);
    } else if (func == SPEFUNC_PUTS) {        
            // puts()
        char *temp = spegetstring(spe, arg & spe->as->lslr);
        ret = puts(temp);
        delete[]temp;
    } else if (func == SPEFUNC_PRINTF) {        
            // vprintf()
        char *fmt = spegetstring(spe, arg & spe->as->lslr);
        addr += 16;
        spe->ls->read4b(addr, &arg);
        ret = spevprintf(spe, fmt, arg & spe->as->lslr);
        delete[]fmt;
    } else {
        printf("## SPE %d unhandled function call(#%d). skip.\n",
                spe->unitid - 1, (int) func);
        ret = arg;
    } spe->ls->write4b(addr, ret);
} 

/**********************************************************************/ 
