/**********************************************************************/  
/*  SimCell: Simulator of Cell/B.E.              Arch Lab. TOKYO TECH */ 
/**********************************************************************/ 
#include "define.h"
    
/**********************************************************************/
SpuInst::SpuInst(Chip * chip, Spe * spe) 
{
    this->chip = chip;
    this->spe = spe;
    this->as = spe->as;
    this->iinfo = chip->iinfo;
    rra = NULL;
    rrb = NULL;
    rrc = NULL;
    rrt = new Register(chip);
}

/**********************************************************************/
SpuInst::~SpuInst() 
{
    delete rra;
    rra = NULL;
    delete rrb;
    rrb = NULL;
    delete rrc;
    rrc = NULL;
    delete rrt;
    rrt = NULL;
}

/**********************************************************************/ 
void SpuInst::exprinthead() 
{
    printf("\n[id %d / inst %lld]\n", spe->unitid, spe->ss->instcnt);
    printf("%5x:", (uint) pc);
    for (int i = 0; i < 4; i++)
        printf(" %02x", (uint) ((ir >> (24 - i * 8)) & 0xff));
    printf("  %s\n", getopname());
    if (type & IT_R_I7)
        printf("imm7 = %d(0x%x)\n", (int) exts32(imm, 7), imm);
    else if (type & IT_R_I8)
        printf("imm8 = %d(0x%x)\n", (int) exts32(imm, 8), imm);
    else if (type & IT_R_I10)
        printf("imm10 = %d(0x%x)\n", (int) exts32(imm, 10), imm);
    else if (type & IT_R_I16)
        printf("imm16 = %d(0x%x)\n", (int) exts32(imm, 16), imm);
    else if (type & IT_R_I18)
        printf("imm18 = %d(0x%x)\n", (int) exts32(imm, 18), imm);
    if (type & IT_R_CA)
        printf("ca = %d(0x%x)\n", ca, ca);
    if ((type & IT_R_RO1) || (type & IT_R_RO2))
        printf("ro = 0x%05x\n", ro);
    if (type & IT_R_RA) {
        printf("ra = r%3d: ", ra);
        rra->print();
    }
    if (type & IT_R_RB) {
        printf("rb = r%3d: ", rb);
        rrb->print();
    }
    if (type & IT_R_RC) {
        printf("rc = r%3d: ", rc);
        rrc->print();
    }
    if ((type & IT_R_RT) || (type & IT_W_RT)) {
        printf("rt = r%3d: ", rt);
        if (type & IT_R_RT)
            rrt->print();
        else
            printf("0x--------------------------------\n");
    }
}

/**********************************************************************/
void SpuInst::exprinttail() 
{
    if (type & IT_W_RT) {
        printf("        => ");
        rrt->print();
    }
}

/**********************************************************************/ 
int SpuInst::isload() 
{
    return ((op >= LQD______) && (op <= LQR______));
}

/**********************************************************************/
int SpuInst::isstore() 
{
    return ((op >= STQD_____) && (op <= STQR_____));
}

/**********************************************************************/
int SpuInst::isbranch() 
{
    return ((op >= BR_______) && (op <= BIHNZ____));
}

/**********************************************************************/
int SpuInst::ishintbranch() 
{
    return ((op >= HBR______) && (op <= HBRR_____));
}

/**********************************************************************/
char *SpuInst::getopname()
{
    return iinfo->getopname(op);
}

/**********************************************************************/ 
void SpuInst::clear() 
{
    rra = NULL;
    rrb = NULL;
    rrc = NULL;
    targ = NOADDR;
    pc = NOADDR;
    nextpc = NOADDR;
    nextstate = SPE_RUN;
}

/**********************************************************************/ 
void SpuInst::decode() 
{
    op = iinfo->getinstid(ir >> 21);
    spuinstattr_t attr = iinfo->getinstattr(op);
    type = attr.type;
    pipe = attr.pipe;
    latency = attr.latency;
    stall = attr.stall;
    nextpc = pc + 4;

    rt = ir & 0x7f;
    ra = (ir >> 7) & 0x7f;
    rb = (ir >> 14) & 0x7f;
    if (type & IT_R_RC) {
        rc = rt;
        rt = (ir >> 21) & 0x7f;
    }

    if (type & IT_W_RT)
        depend = rt;
    else
        depend = 128;
    
    if (type & IT_R_I18)
        imm = (ir >> 7) & 0x3ffff;
    else if (type & IT_R_I16)
        imm = (ir >> 7) & 0xffff;
    else if (type & IT_R_I10)
        imm = (ir >> 14) & 0x3ff;
    else if (type & IT_R_I8)
        imm = (ir >> 14) & 0xff;
    else if (type & IT_R_I7)
        imm = (ir >> 14) & 0x7f;

    ca = ra;

    if (type & IT_R_RO1)
        ro = pc + exts32(((ir & 0x7f) | ((ir >> 7) & 0x180)) << 2, 11);
    else if (type & IT_R_RO2)
        ro = pc + exts32(((ir & 0x7f) | ((ir >> 16) & 0x180)) << 2, 11);
    
    if (type & IT_R_RA)
        rra = as->reg[ra];
    if (type & IT_R_RB)
        rrb = as->reg[rb];
    if (type & IT_R_RC)
        rrc = as->reg[rc];
    if (type & IT_R_RT)
        rrt->put128t(as->reg[rt]->get128t());
}

/**********************************************************************/ 
int SpuInstinfo::predecode(int ir) 
{
    int opc;
    
    // opcode = 11bit
    opc = ir & 0x7ff;
    if      (opc == 0x1c4) return LQX______;
    else if (opc == 0x144) return STQX_____;
    else if (opc == 0x1f4) return CBD______;
    else if (opc == 0x1d4) return CBX______;
    else if (opc == 0x1f5) return CHD______;
    else if (opc == 0x1d5) return CHX______;
    else if (opc == 0x1f6) return CWD______;
    else if (opc == 0x1d6) return CWX______;
    else if (opc == 0x1f7) return CDD______;
    else if (opc == 0x1d7) return CDX______;
    else if (opc == 0x0c8) return AH_______;
    else if (opc == 0x0c0) return A________;
    else if (opc == 0x048) return SFH______;
    else if (opc == 0x040) return SF_______;
    else if (opc == 0x340) return ADDX_____;
    else if (opc == 0x0c2) return CG_______;
    else if (opc == 0x342) return CGX______;
    else if (opc == 0x341) return SFX______;
    else if (opc == 0x042) return BG_______;
    else if (opc == 0x343) return BGX______;
    else if (opc == 0x3c4) return MPY______;
    else if (opc == 0x3cc) return MPYU_____;
    else if (opc == 0x3c5) return MPYH_____;
    else if (opc == 0x3c7) return MPYS_____;
    else if (opc == 0x3c6) return MPYHH____;
    else if (opc == 0x346) return MPYHHA___;
    else if (opc == 0x3ce) return MPYHHU___;
    else if (opc == 0x34e) return MPYHHAU__;
    else if (opc == 0x2a5) return CLZ______;
    else if (opc == 0x2b4) return CNTB_____;
    else if (opc == 0x1b6) return FSMB_____;
    else if (opc == 0x1b5) return FSMH_____;
    else if (opc == 0x1b4) return FSM______;
    else if (opc == 0x1b2) return GBB______;
    else if (opc == 0x1b1) return GBH______;
    else if (opc == 0x1b0) return GB_______;
    else if (opc == 0x0d3) return AVGB_____;
    else if (opc == 0x053) return ABSDB____;
    else if (opc == 0x253) return SUMB_____;
    else if (opc == 0x2b6) return XSBH_____;
    else if (opc == 0x2ae) return XSHW_____;
    else if (opc == 0x2a6) return XSWD_____;
    else if (opc == 0x0c1) return AND______;
    else if (opc == 0x2c1) return ANDC_____;
    else if (opc == 0x041) return OR_______;
    else if (opc == 0x2c9) return ORC______;
    else if (opc == 0x1f0) return ORX______;
    else if (opc == 0x241) return XOR______;
    else if (opc == 0x0c9) return NAND_____;
    else if (opc == 0x049) return NOR______;
    else if (opc == 0x249) return EQV______;
    else if (opc == 0x05f) return SHLH_____;
    else if (opc == 0x07f) return SHLHI____;
    else if (opc == 0x05b) return SHL______;
    else if (opc == 0x07b) return SHLI_____;
    else if (opc == 0x1db) return SHLQBI___;
    else if (opc == 0x1fb) return SHLQBII__;
    else if (opc == 0x1df) return SHLQBY___;
    else if (opc == 0x1ff) return SHLQBYI__;
    else if (opc == 0x1cf) return SHLQBYBI_;
    else if (opc == 0x05c) return ROTH_____;
    else if (opc == 0x07c) return ROTHI____;
    else if (opc == 0x058) return ROT______;
    else if (opc == 0x078) return ROTI_____;
    else if (opc == 0x1dc) return ROTQBY___;
    else if (opc == 0x1fc) return ROTQBYI__;
    else if (opc == 0x1cc) return ROTQBYBI_;
    else if (opc == 0x1d8) return ROTQBI___;
    else if (opc == 0x1f8) return ROTQBII__;
    else if (opc == 0x05d) return ROTHM____;
    else if (opc == 0x07d) return ROTHMI___;
    else if (opc == 0x059) return ROTM_____;
    else if (opc == 0x079) return ROTMI____;
    else if (opc == 0x1dd) return ROTQMBY__;
    else if (opc == 0x1fd) return ROTQMBYI_;
    else if (opc == 0x1cd) return ROTQMBYBI;
    else if (opc == 0x1d9) return ROTQMBI__;
    else if (opc == 0x1f9) return ROTQMBII_;
    else if (opc == 0x05e) return ROTMAH___;
    else if (opc == 0x07e) return ROTMAHI__;
    else if (opc == 0x05a) return ROTMA____;
    else if (opc == 0x07a) return ROTMAI___;
    else if (opc == 0x3d8) return HEQ______;
    else if (opc == 0x258) return HGT______;
    else if (opc == 0x2d8) return HLGT_____;
    else if (opc == 0x3d0) return CEQB_____;
    else if (opc == 0x3c8) return CEQH_____;
    else if (opc == 0x3c0) return CEQ______;
    else if (opc == 0x250) return CGTB_____;
    else if (opc == 0x248) return CGTH_____;
    else if (opc == 0x240) return CGT______;
    else if (opc == 0x2d0) return CLGTB____;
    else if (opc == 0x2c8) return CLGTH____;
    else if (opc == 0x2c0) return CLGT_____;
    else if (opc == 0x1a8) return BI_______;
    else if (opc == 0x1aa) return IRET_____;
    else if (opc == 0x1ab) return BISLED___;
    else if (opc == 0x1a9) return BISL_____;
    else if (opc == 0x128) return BIZ______;
    else if (opc == 0x129) return BINZ_____;
    else if (opc == 0x12a) return BIHZ_____;
    else if (opc == 0x12b) return BIHNZ____;
    else if (opc == 0x1ac) return HBR______;
    else if (opc == 0x2c4) return FA_______;
    else if (opc == 0x2cc) return DFA______;
    else if (opc == 0x2c5) return FS_______;
    else if (opc == 0x2cd) return DFS______;
    else if (opc == 0x2c6) return FM_______;
    else if (opc == 0x2ce) return DFM______;
    else if (opc == 0x35c) return DFMA_____;
    else if (opc == 0x35e) return DFNMS____;
    else if (opc == 0x35d) return DFMS_____;
    else if (opc == 0x35f) return DFNMA____;
    else if (opc == 0x1b8) return FREST____;
    else if (opc == 0x1b9) return FRSQEST__;
    else if (opc == 0x3d4) return FI_______;
    else if (opc == 0x3b9) return FRDS_____;
    else if (opc == 0x3b8) return FESD_____;
    else if (opc == 0x3c3) return DFCEQ____;
    else if (opc == 0x3cb) return DFCMEQ___;
    else if (opc == 0x2c3) return DFCGT____;
    else if (opc == 0x2cb) return DFCMGT___;
    else if (opc == 0x3bf) return DFTSV____;
    else if (opc == 0x3c2) return FCEQ_____;
    else if (opc == 0x3ca) return FCMEQ____;
    else if (opc == 0x2c2) return FCGT_____;
    else if (opc == 0x2ca) return FCMGT____;
    else if (opc == 0x3ba) return FSCRWR___;
    else if (opc == 0x398) return FSCRRD___;
    else if (opc == 0x000) return STOP_____;
    else if (opc == 0x140) return STOPD____;
    else if (opc == 0x001) return LNOP_____;
    else if (opc == 0x201) return NOP______;
    else if (opc == 0x002) return SYNC_____;
    else if (opc == 0x003) return DSYNC____;
    else if (opc == 0x00c) return MFSPR____;
    else if (opc == 0x10c) return MTSPR____;
    else if (opc == 0x00d) return RDCH_____;
    else if (opc == 0x10d) return WRCH_____;
    else if (opc == 0x00f) return RCHCNT___;

    // opcode = 10bit
    opc = (ir >> 1) & 0x3ff;
    if      (opc == 0x1da) return CSFLT____;
    else if (opc == 0x1d8) return CFLTS____;
    else if (opc == 0x1db) return CUFLT____;
    else if (opc == 0x1d9) return CFLTU____;

    // opcode = 9bit
    opc = (ir >> 2) & 0x1ff;
    if      (opc == 0x061) return LQA______;
    else if (opc == 0x067) return LQR______;
    else if (opc == 0x041) return STQA_____;
    else if (opc == 0x047) return STQR_____;
    else if (opc == 0x083) return ILH______;
    else if (opc == 0x082) return ILHU_____;
    else if (opc == 0x081) return IL_______;
    else if (opc == 0x0c1) return IOHL_____;
    else if (opc == 0x065) return FSMBI____;
    else if (opc == 0x064) return BR_______;
    else if (opc == 0x060) return BRA______;
    else if (opc == 0x066) return BRSL_____;
    else if (opc == 0x062) return BRASL____;
    else if (opc == 0x042) return BRNZ_____;
    else if (opc == 0x040) return BRZ______;
    else if (opc == 0x046) return BRHNZ____;
    else if (opc == 0x044) return BRHZ_____;

    // opcode = 8bit
    opc = (ir >> 3) & 0xff;
    if      (opc == 0x34) return LQD______;
    else if (opc == 0x24) return STQD_____;
    else if (opc == 0x1d) return AHI______;
    else if (opc == 0x1c) return AI_______;
    else if (opc == 0x0d) return SFHI_____;
    else if (opc == 0x0c) return SFI______;
    else if (opc == 0x74) return MPYI_____;
    else if (opc == 0x75) return MPYUI____;
    else if (opc == 0x16) return ANDBI____;
    else if (opc == 0x15) return ANDHI____;
    else if (opc == 0x14) return ANDI_____;
    else if (opc == 0x06) return ORBI_____;
    else if (opc == 0x05) return ORHI_____;
    else if (opc == 0x04) return ORI______;
    else if (opc == 0x46) return XORBI____;
    else if (opc == 0x45) return XORHI____;
    else if (opc == 0x44) return XORI_____;
    else if (opc == 0x7f) return HEQI_____;
    else if (opc == 0x4f) return HGTI_____;
    else if (opc == 0x5f) return HLGTI____;
    else if (opc == 0x7e) return CEQBI____;
    else if (opc == 0x7d) return CEQHI____;
    else if (opc == 0x7c) return CEQI_____;
    else if (opc == 0x4e) return CGTBI____;
    else if (opc == 0x4d) return CGTHI____;
    else if (opc == 0x4c) return CGTI_____;
    else if (opc == 0x5e) return CLGTBI___;
    else if (opc == 0x5d) return CLGTHI___;
    else if (opc == 0x5c) return CLGTI____;

    // opcode = 7bit
    opc = (ir >> 4) & 0x7f;
    if      (opc == 0x21) return ILA______;
    else if (opc == 0x08) return HBRA_____;
    else if (opc == 0x09) return HBRR_____;

    // opcode = 4bit
    opc = (ir >> 7) & 0xf;
    if      (opc == 0xc) return MPYA_____;
    else if (opc == 0x8) return SELB_____;
    else if (opc == 0xb) return SHUFB____;
    else if (opc == 0xe) return FMA______;
    else if (opc == 0xd) return FNMS_____;
    else if (opc == 0xf) return FMS______;

    return UNDEF____;
}

/**********************************************************************/ 
void SpuInstinfo::setattr()
{
    id_to_attr[LQD______].name    = "lqd";
    id_to_attr[LQD______].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[LQD______].pipe    = PIPE_ODD;
    id_to_attr[LQD______].latency = 6;
    id_to_attr[LQD______].stall   = 0;
    id_to_attr[LQX______].name    = "lqx";
    id_to_attr[LQX______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[LQX______].pipe    = PIPE_ODD;
    id_to_attr[LQX______].latency = 6;
    id_to_attr[LQX______].stall   = 0;
    id_to_attr[LQA______].name    = "lqa";
    id_to_attr[LQA______].type    = IT_W_RT | IT_R_I16;
    id_to_attr[LQA______].pipe    = PIPE_ODD;
    id_to_attr[LQA______].latency = 6;
    id_to_attr[LQA______].stall   = 0;
    id_to_attr[LQR______].name    = "lqr";
    id_to_attr[LQR______].type    = IT_W_RT | IT_R_I16;
    id_to_attr[LQR______].pipe    = PIPE_ODD;
    id_to_attr[LQR______].latency = 6;
    id_to_attr[LQR______].stall   = 0;
    id_to_attr[STQD_____].name    = "stqd";
    id_to_attr[STQD_____].type    = IT_R_RA | IT_R_RT | IT_R_I10;
    id_to_attr[STQD_____].pipe    = PIPE_ODD;
    id_to_attr[STQD_____].latency = 6;
    id_to_attr[STQD_____].stall   = 0;
    id_to_attr[STQX_____].name    = "stqx";
    id_to_attr[STQX_____].type    = IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[STQX_____].pipe    = PIPE_ODD;
    id_to_attr[STQX_____].latency = 6;
    id_to_attr[STQX_____].stall   = 0;
    id_to_attr[STQA_____].name    = "stqa";
    id_to_attr[STQA_____].type    = IT_R_RT | IT_R_I16;
    id_to_attr[STQA_____].pipe    = PIPE_ODD;
    id_to_attr[STQA_____].latency = 6;
    id_to_attr[STQA_____].stall   = 0;
    id_to_attr[STQR_____].name    = "stqr";
    id_to_attr[STQR_____].type    = IT_R_RT | IT_R_I16;
    id_to_attr[STQR_____].pipe    = PIPE_ODD;
    id_to_attr[STQR_____].latency = 6;
    id_to_attr[STQR_____].stall   = 0;
    id_to_attr[CBD______].name    = "cbd";
    id_to_attr[CBD______].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[CBD______].pipe    = PIPE_ODD;
    id_to_attr[CBD______].latency = 4;
    id_to_attr[CBD______].stall   = 0;
    id_to_attr[CBX______].name    = "cbx";
    id_to_attr[CBX______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CBX______].pipe    = PIPE_ODD;
    id_to_attr[CBX______].latency = 4;
    id_to_attr[CBX______].stall   = 0;
    id_to_attr[CHD______].name    = "chd";
    id_to_attr[CHD______].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[CHD______].pipe    = PIPE_ODD;
    id_to_attr[CHD______].latency = 4;
    id_to_attr[CHD______].stall   = 0;
    id_to_attr[CHX______].name    = "chx";
    id_to_attr[CHX______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CHX______].pipe    = PIPE_ODD;
    id_to_attr[CHX______].latency = 4;
    id_to_attr[CHX______].stall   = 0;
    id_to_attr[CWD______].name    = "cwd";
    id_to_attr[CWD______].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[CWD______].pipe    = PIPE_ODD;
    id_to_attr[CWD______].latency = 4;
    id_to_attr[CWD______].stall   = 0;
    id_to_attr[CWX______].name    = "cwx";
    id_to_attr[CWX______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CWX______].pipe    = PIPE_ODD;
    id_to_attr[CWX______].latency = 4;
    id_to_attr[CWX______].stall   = 0;
    id_to_attr[CDD______].name    = "cdd";
    id_to_attr[CDD______].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[CDD______].pipe    = PIPE_ODD;
    id_to_attr[CDD______].latency = 4;
    id_to_attr[CDD______].stall   = 0;
    id_to_attr[CDX______].name    = "cdx";
    id_to_attr[CDX______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CDX______].pipe    = PIPE_ODD;
    id_to_attr[CDX______].latency = 4;
    id_to_attr[CDX______].stall   = 0;
    id_to_attr[ILH______].name    = "ilh";
    id_to_attr[ILH______].type    = IT_W_RT | IT_R_I16;
    id_to_attr[ILH______].pipe    = PIPE_EVEN;
    id_to_attr[ILH______].latency = 2;
    id_to_attr[ILH______].stall   = 0;
    id_to_attr[ILHU_____].name    = "ilhu";
    id_to_attr[ILHU_____].type    = IT_W_RT | IT_R_I16;
    id_to_attr[ILHU_____].pipe    = PIPE_EVEN;
    id_to_attr[ILHU_____].latency = 2;
    id_to_attr[ILHU_____].stall   = 0;
    id_to_attr[IL_______].name    = "il";
    id_to_attr[IL_______].type    = IT_W_RT | IT_R_I16;
    id_to_attr[IL_______].pipe    = PIPE_EVEN;
    id_to_attr[IL_______].latency = 2;
    id_to_attr[IL_______].stall   = 0;
    id_to_attr[ILA______].name    = "ila";
    id_to_attr[ILA______].type    = IT_W_RT | IT_R_I18;
    id_to_attr[ILA______].pipe    = PIPE_EVEN;
    id_to_attr[ILA______].latency = 2;
    id_to_attr[ILA______].stall   = 0;
    id_to_attr[IOHL_____].name    = "iohl";
    id_to_attr[IOHL_____].type    = IT_W_RT | IT_R_RT | IT_R_I16;
    id_to_attr[IOHL_____].pipe    = PIPE_EVEN;
    id_to_attr[IOHL_____].latency = 2;
    id_to_attr[IOHL_____].stall   = 0;
    id_to_attr[FSMBI____].name    = "fsmbi";
    id_to_attr[FSMBI____].type    = IT_W_RT | IT_R_I16;
    id_to_attr[FSMBI____].pipe    = PIPE_ODD;
    id_to_attr[FSMBI____].latency = 4;
    id_to_attr[FSMBI____].stall   = 0;
    id_to_attr[AH_______].name    = "ah";
    id_to_attr[AH_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[AH_______].pipe    = PIPE_EVEN;
    id_to_attr[AH_______].latency = 2;
    id_to_attr[AH_______].stall   = 0;
    id_to_attr[AHI______].name    = "ahi";
    id_to_attr[AHI______].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[AHI______].pipe    = PIPE_EVEN;
    id_to_attr[AHI______].latency = 2;
    id_to_attr[AHI______].stall   = 0;
    id_to_attr[A________].name    = "a";
    id_to_attr[A________].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[A________].pipe    = PIPE_EVEN;
    id_to_attr[A________].latency = 2;
    id_to_attr[A________].stall   = 0;
    id_to_attr[AI_______].name    = "ai";
    id_to_attr[AI_______].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[AI_______].pipe    = PIPE_EVEN;
    id_to_attr[AI_______].latency = 2;
    id_to_attr[AI_______].stall   = 0;
    id_to_attr[SFH______].name    = "sfh";
    id_to_attr[SFH______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SFH______].pipe    = PIPE_EVEN;
    id_to_attr[SFH______].latency = 2;
    id_to_attr[SFH______].stall   = 0;
    id_to_attr[SFHI_____].name    = "sfhi";
    id_to_attr[SFHI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[SFHI_____].pipe    = PIPE_EVEN;
    id_to_attr[SFHI_____].latency = 2;
    id_to_attr[SFHI_____].stall   = 0;
    id_to_attr[SF_______].name    = "sf";
    id_to_attr[SF_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SF_______].pipe    = PIPE_EVEN;
    id_to_attr[SF_______].latency = 2;
    id_to_attr[SF_______].stall   = 0;
    id_to_attr[SFI______].name    = "sfi";
    id_to_attr[SFI______].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[SFI______].pipe    = PIPE_EVEN;
    id_to_attr[SFI______].latency = 2;
    id_to_attr[SFI______].stall   = 0;
    id_to_attr[ADDX_____].name    = "addx";
    id_to_attr[ADDX_____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[ADDX_____].pipe    = PIPE_EVEN;
    id_to_attr[ADDX_____].latency = 2;
    id_to_attr[ADDX_____].stall   = 0;
    id_to_attr[CG_______].name    = "cg";
    id_to_attr[CG_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CG_______].pipe    = PIPE_EVEN;
    id_to_attr[CG_______].latency = 2;
    id_to_attr[CG_______].stall   = 0;
    id_to_attr[CGX______].name    = "cgx";
    id_to_attr[CGX______].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[CGX______].pipe    = PIPE_EVEN;
    id_to_attr[CGX______].latency = 2;
    id_to_attr[CGX______].stall   = 0;
    id_to_attr[SFX______].name    = "sfx";
    id_to_attr[SFX______].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[SFX______].pipe    = PIPE_EVEN;
    id_to_attr[SFX______].latency = 2;
    id_to_attr[SFX______].stall   = 0;
    id_to_attr[BG_______].name    = "bg";
    id_to_attr[BG_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[BG_______].pipe    = PIPE_EVEN;
    id_to_attr[BG_______].latency = 2;
    id_to_attr[BG_______].stall   = 0;
    id_to_attr[BGX______].name    = "bgx";
    id_to_attr[BGX______].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[BGX______].pipe    = PIPE_EVEN;
    id_to_attr[BGX______].latency = 2;
    id_to_attr[BGX______].stall   = 0;
    id_to_attr[MPY______].name    = "mpy";
    id_to_attr[MPY______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[MPY______].pipe    = PIPE_EVEN;
    id_to_attr[MPY______].latency = 7;
    id_to_attr[MPY______].stall   = 0;
    id_to_attr[MPYU_____].name    = "mpyu";
    id_to_attr[MPYU_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[MPYU_____].pipe    = PIPE_EVEN;
    id_to_attr[MPYU_____].latency = 7;
    id_to_attr[MPYU_____].stall   = 0;
    id_to_attr[MPYI_____].name    = "mpyi";
    id_to_attr[MPYI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[MPYI_____].pipe    = PIPE_EVEN;
    id_to_attr[MPYI_____].latency = 7;
    id_to_attr[MPYI_____].stall   = 0;
    id_to_attr[MPYUI____].name    = "mpyui";
    id_to_attr[MPYUI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[MPYUI____].pipe    = PIPE_EVEN;
    id_to_attr[MPYUI____].latency = 7;
    id_to_attr[MPYUI____].stall   = 0;
    id_to_attr[MPYA_____].name    = "mpya";
    id_to_attr[MPYA_____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RC;
    id_to_attr[MPYA_____].pipe    = PIPE_EVEN;
    id_to_attr[MPYA_____].latency = 7;
    id_to_attr[MPYA_____].stall   = 0;
    id_to_attr[MPYH_____].name    = "mpyh";
    id_to_attr[MPYH_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[MPYH_____].pipe    = PIPE_EVEN;
    id_to_attr[MPYH_____].latency = 7;
    id_to_attr[MPYH_____].stall   = 0;
    id_to_attr[MPYS_____].name    = "mpys";
    id_to_attr[MPYS_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[MPYS_____].pipe    = PIPE_EVEN;
    id_to_attr[MPYS_____].latency = 7;
    id_to_attr[MPYS_____].stall   = 0;
    id_to_attr[MPYHH____].name    = "mpyhh";
    id_to_attr[MPYHH____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[MPYHH____].pipe    = PIPE_EVEN;
    id_to_attr[MPYHH____].latency = 7;
    id_to_attr[MPYHH____].stall   = 0;
    id_to_attr[MPYHHA___].name    = "mpyhha";
    id_to_attr[MPYHHA___].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[MPYHHA___].pipe    = PIPE_EVEN;
    id_to_attr[MPYHHA___].latency = 7;
    id_to_attr[MPYHHA___].stall   = 0;
    id_to_attr[MPYHHU___].name    = "mpyhhu";
    id_to_attr[MPYHHU___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[MPYHHU___].pipe    = PIPE_EVEN;
    id_to_attr[MPYHHU___].latency = 7;
    id_to_attr[MPYHHU___].stall   = 0;
    id_to_attr[MPYHHAU__].name    = "mpyhhau";
    id_to_attr[MPYHHAU__].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[MPYHHAU__].pipe    = PIPE_EVEN;
    id_to_attr[MPYHHAU__].latency = 7;
    id_to_attr[MPYHHAU__].stall   = 0;
    id_to_attr[CLZ______].name    = "clz";
    id_to_attr[CLZ______].type    = IT_W_RT | IT_R_RA;
    id_to_attr[CLZ______].pipe    = PIPE_EVEN;
    id_to_attr[CLZ______].latency = 2;
    id_to_attr[CLZ______].stall   = 0;
    id_to_attr[CNTB_____].name    = "cntb";
    id_to_attr[CNTB_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[CNTB_____].pipe    = PIPE_EVEN;
    id_to_attr[CNTB_____].latency = 4;
    id_to_attr[CNTB_____].stall   = 0;
    id_to_attr[FSMB_____].name    = "fsmb";
    id_to_attr[FSMB_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[FSMB_____].pipe    = PIPE_ODD;
    id_to_attr[FSMB_____].latency = 4;
    id_to_attr[FSMB_____].stall   = 0;
    id_to_attr[FSMH_____].name    = "fsmh";
    id_to_attr[FSMH_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[FSMH_____].pipe    = PIPE_ODD;
    id_to_attr[FSMH_____].latency = 4;
    id_to_attr[FSMH_____].stall   = 0;
    id_to_attr[FSM______].name    = "fsm";
    id_to_attr[FSM______].type    = IT_W_RT | IT_R_RA;
    id_to_attr[FSM______].pipe    = PIPE_ODD;
    id_to_attr[FSM______].latency = 4;
    id_to_attr[FSM______].stall   = 0;
    id_to_attr[GBB______].name    = "gbb";
    id_to_attr[GBB______].type    = IT_W_RT | IT_R_RA;
    id_to_attr[GBB______].pipe    = PIPE_ODD;
    id_to_attr[GBB______].latency = 4;
    id_to_attr[GBB______].stall   = 0;
    id_to_attr[GBH______].name    = "gbh";
    id_to_attr[GBH______].type    = IT_W_RT | IT_R_RA;
    id_to_attr[GBH______].pipe    = PIPE_ODD;
    id_to_attr[GBH______].latency = 4;
    id_to_attr[GBH______].stall   = 0;
    id_to_attr[GB_______].name    = "gb";
    id_to_attr[GB_______].type    = IT_W_RT | IT_R_RA;
    id_to_attr[GB_______].pipe    = PIPE_ODD;
    id_to_attr[GB_______].latency = 4;
    id_to_attr[GB_______].stall   = 0;
    id_to_attr[AVGB_____].name    = "avgb";
    id_to_attr[AVGB_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[AVGB_____].pipe    = PIPE_EVEN;
    id_to_attr[AVGB_____].latency = 4;
    id_to_attr[AVGB_____].stall   = 0;
    id_to_attr[ABSDB____].name    = "absdb";
    id_to_attr[ABSDB____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ABSDB____].pipe    = PIPE_EVEN;
    id_to_attr[ABSDB____].latency = 4;
    id_to_attr[ABSDB____].stall   = 0;
    id_to_attr[SUMB_____].name    = "sumb";
    id_to_attr[SUMB_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SUMB_____].pipe    = PIPE_EVEN;
    id_to_attr[SUMB_____].latency = 4;
    id_to_attr[SUMB_____].stall   = 0;
    id_to_attr[XSBH_____].name    = "xsbh";
    id_to_attr[XSBH_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[XSBH_____].pipe    = PIPE_EVEN;
    id_to_attr[XSBH_____].latency = 2;
    id_to_attr[XSBH_____].stall   = 0;
    id_to_attr[XSHW_____].name    = "xshw";
    id_to_attr[XSHW_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[XSHW_____].pipe    = PIPE_EVEN;
    id_to_attr[XSHW_____].latency = 2;
    id_to_attr[XSHW_____].stall   = 0;
    id_to_attr[XSWD_____].name    = "xswd";
    id_to_attr[XSWD_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[XSWD_____].pipe    = PIPE_EVEN;
    id_to_attr[XSWD_____].latency = 2;
    id_to_attr[XSWD_____].stall   = 0;
    id_to_attr[AND______].name    = "and";
    id_to_attr[AND______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[AND______].pipe    = PIPE_EVEN;
    id_to_attr[AND______].latency = 2;
    id_to_attr[AND______].stall   = 0;
    id_to_attr[ANDC_____].name    = "andc";
    id_to_attr[ANDC_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ANDC_____].pipe    = PIPE_EVEN;
    id_to_attr[ANDC_____].latency = 2;
    id_to_attr[ANDC_____].stall   = 0;
    id_to_attr[ANDBI____].name    = "andbi";
    id_to_attr[ANDBI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[ANDBI____].pipe    = PIPE_EVEN;
    id_to_attr[ANDBI____].latency = 2;
    id_to_attr[ANDBI____].stall   = 0;
    id_to_attr[ANDHI____].name    = "andhi";
    id_to_attr[ANDHI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[ANDHI____].pipe    = PIPE_EVEN;
    id_to_attr[ANDHI____].latency = 2;
    id_to_attr[ANDHI____].stall   = 0;
    id_to_attr[ANDI_____].name    = "andi";
    id_to_attr[ANDI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[ANDI_____].pipe    = PIPE_EVEN;
    id_to_attr[ANDI_____].latency = 2;
    id_to_attr[ANDI_____].stall   = 0;
    id_to_attr[OR_______].name    = "or";
    id_to_attr[OR_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[OR_______].pipe    = PIPE_EVEN;
    id_to_attr[OR_______].latency = 2;
    id_to_attr[OR_______].stall   = 0;
    id_to_attr[ORC______].name    = "orc";
    id_to_attr[ORC______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ORC______].pipe    = PIPE_EVEN;
    id_to_attr[ORC______].latency = 2;
    id_to_attr[ORC______].stall   = 0;
    id_to_attr[ORBI_____].name    = "orbi";
    id_to_attr[ORBI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[ORBI_____].pipe    = PIPE_EVEN;
    id_to_attr[ORBI_____].latency = 2;
    id_to_attr[ORBI_____].stall   = 0;
    id_to_attr[ORHI_____].name    = "orhi";
    id_to_attr[ORHI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[ORHI_____].pipe    = PIPE_EVEN;
    id_to_attr[ORHI_____].latency = 2;
    id_to_attr[ORHI_____].stall   = 0;
    id_to_attr[ORI______].name    = "ori";
    id_to_attr[ORI______].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[ORI______].pipe    = PIPE_EVEN;
    id_to_attr[ORI______].latency = 2;
    id_to_attr[ORI______].stall   = 0;
    id_to_attr[ORX______].name    = "orx";
    id_to_attr[ORX______].type    = IT_W_RT | IT_R_RA;
    id_to_attr[ORX______].pipe    = PIPE_ODD;
    id_to_attr[ORX______].latency = 4;
    id_to_attr[ORX______].stall   = 0;
    id_to_attr[XOR______].name    = "xor";
    id_to_attr[XOR______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[XOR______].pipe    = PIPE_EVEN;
    id_to_attr[XOR______].latency = 2;
    id_to_attr[XOR______].stall   = 0;
    id_to_attr[XORBI____].name    = "xorbi";
    id_to_attr[XORBI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[XORBI____].pipe    = PIPE_EVEN;
    id_to_attr[XORBI____].latency = 2;
    id_to_attr[XORBI____].stall   = 0;
    id_to_attr[XORHI____].name    = "xorhi";
    id_to_attr[XORHI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[XORHI____].pipe    = PIPE_EVEN;
    id_to_attr[XORHI____].latency = 2;
    id_to_attr[XORHI____].stall   = 0;
    id_to_attr[XORI_____].name    = "xori";
    id_to_attr[XORI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[XORI_____].pipe    = PIPE_EVEN;
    id_to_attr[XORI_____].latency = 2;
    id_to_attr[XORI_____].stall   = 0;
    id_to_attr[NAND_____].name    = "nand";
    id_to_attr[NAND_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[NAND_____].pipe    = PIPE_EVEN;
    id_to_attr[NAND_____].latency = 2;
    id_to_attr[NAND_____].stall   = 0;
    id_to_attr[NOR______].name    = "nor";
    id_to_attr[NOR______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[NOR______].pipe    = PIPE_EVEN;
    id_to_attr[NOR______].latency = 2;
    id_to_attr[NOR______].stall   = 0;
    id_to_attr[EQV______].name    = "eqv";
    id_to_attr[EQV______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[EQV______].pipe    = PIPE_EVEN;
    id_to_attr[EQV______].latency = 2;
    id_to_attr[EQV______].stall   = 0;
    id_to_attr[SELB_____].name    = "selb";
    id_to_attr[SELB_____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RC;
    id_to_attr[SELB_____].pipe    = PIPE_EVEN;
    id_to_attr[SELB_____].latency = 2;
    id_to_attr[SELB_____].stall   = 0;
    id_to_attr[SHUFB____].name    = "shufb";
    id_to_attr[SHUFB____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RC;
    id_to_attr[SHUFB____].pipe    = PIPE_ODD;
    id_to_attr[SHUFB____].latency = 4;
    id_to_attr[SHUFB____].stall   = 0;
    id_to_attr[SHLH_____].name    = "shlh";
    id_to_attr[SHLH_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SHLH_____].pipe    = PIPE_EVEN;
    id_to_attr[SHLH_____].latency = 4;
    id_to_attr[SHLH_____].stall   = 0;
    id_to_attr[SHLHI____].name    = "shlhi";
    id_to_attr[SHLHI____].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[SHLHI____].pipe    = PIPE_EVEN;
    id_to_attr[SHLHI____].latency = 4;
    id_to_attr[SHLHI____].stall   = 0;
    id_to_attr[SHL______].name    = "shl";
    id_to_attr[SHL______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SHL______].pipe    = PIPE_EVEN;
    id_to_attr[SHL______].latency = 4;
    id_to_attr[SHL______].stall   = 0;
    id_to_attr[SHLI_____].name    = "shli";
    id_to_attr[SHLI_____].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[SHLI_____].pipe    = PIPE_EVEN;
    id_to_attr[SHLI_____].latency = 4;
    id_to_attr[SHLI_____].stall   = 0;
    id_to_attr[SHLQBI___].name    = "shlqbi";
    id_to_attr[SHLQBI___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SHLQBI___].pipe    = PIPE_ODD;
    id_to_attr[SHLQBI___].latency = 4;
    id_to_attr[SHLQBI___].stall   = 0;
    id_to_attr[SHLQBII__].name    = "shlqbii";
    id_to_attr[SHLQBII__].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[SHLQBII__].pipe    = PIPE_ODD;
    id_to_attr[SHLQBII__].latency = 4;
    id_to_attr[SHLQBII__].stall   = 0;
    id_to_attr[SHLQBY___].name    = "shlqby";
    id_to_attr[SHLQBY___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SHLQBY___].pipe    = PIPE_ODD;
    id_to_attr[SHLQBY___].latency = 4;
    id_to_attr[SHLQBY___].stall   = 0;
    id_to_attr[SHLQBYI__].name    = "shlqbyi";
    id_to_attr[SHLQBYI__].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[SHLQBYI__].pipe    = PIPE_ODD;
    id_to_attr[SHLQBYI__].latency = 4;
    id_to_attr[SHLQBYI__].stall   = 0;
    id_to_attr[SHLQBYBI_].name    = "shlqbybi";
    id_to_attr[SHLQBYBI_].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[SHLQBYBI_].pipe    = PIPE_ODD;
    id_to_attr[SHLQBYBI_].latency = 4;
    id_to_attr[SHLQBYBI_].stall   = 0;
    id_to_attr[ROTH_____].name    = "roth";
    id_to_attr[ROTH_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTH_____].pipe    = PIPE_EVEN;
    id_to_attr[ROTH_____].latency = 4;
    id_to_attr[ROTH_____].stall   = 0;
    id_to_attr[ROTHI____].name    = "rothi";
    id_to_attr[ROTHI____].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTHI____].pipe    = PIPE_EVEN;
    id_to_attr[ROTHI____].latency = 4;
    id_to_attr[ROTHI____].stall   = 0;
    id_to_attr[ROT______].name    = "rot";
    id_to_attr[ROT______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROT______].pipe    = PIPE_EVEN;
    id_to_attr[ROT______].latency = 4;
    id_to_attr[ROT______].stall   = 0;
    id_to_attr[ROTI_____].name    = "roti";
    id_to_attr[ROTI_____].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTI_____].pipe    = PIPE_EVEN;
    id_to_attr[ROTI_____].latency = 4;
    id_to_attr[ROTI_____].stall   = 0;
    id_to_attr[ROTQBY___].name    = "rotqby";
    id_to_attr[ROTQBY___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTQBY___].pipe    = PIPE_ODD;
    id_to_attr[ROTQBY___].latency = 4;
    id_to_attr[ROTQBY___].stall   = 0;
    id_to_attr[ROTQBYI__].name    = "rotqbyi";
    id_to_attr[ROTQBYI__].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTQBYI__].pipe    = PIPE_ODD;
    id_to_attr[ROTQBYI__].latency = 4;
    id_to_attr[ROTQBYI__].stall   = 0;
    id_to_attr[ROTQBYBI_].name    = "rotqbybi";
    id_to_attr[ROTQBYBI_].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTQBYBI_].pipe    = PIPE_ODD;
    id_to_attr[ROTQBYBI_].latency = 4;
    id_to_attr[ROTQBYBI_].stall   = 0;
    id_to_attr[ROTQBI___].name    = "rotqbi";
    id_to_attr[ROTQBI___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTQBI___].pipe    = PIPE_ODD;
    id_to_attr[ROTQBI___].latency = 4;
    id_to_attr[ROTQBI___].stall   = 0;
    id_to_attr[ROTQBII__].name    = "rotqbii";
    id_to_attr[ROTQBII__].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTQBII__].pipe    = PIPE_ODD;
    id_to_attr[ROTQBII__].latency = 4;
    id_to_attr[ROTQBII__].stall   = 0;
    id_to_attr[ROTHM____].name    = "rothm";
    id_to_attr[ROTHM____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTHM____].pipe    = PIPE_EVEN;
    id_to_attr[ROTHM____].latency = 4;
    id_to_attr[ROTHM____].stall   = 0;
    id_to_attr[ROTHMI___].name    = "rothmi";
    id_to_attr[ROTHMI___].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTHMI___].pipe    = PIPE_EVEN;
    id_to_attr[ROTHMI___].latency = 4;
    id_to_attr[ROTHMI___].stall   = 0;
    id_to_attr[ROTM_____].name    = "rotm";
    id_to_attr[ROTM_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTM_____].pipe    = PIPE_EVEN;
    id_to_attr[ROTM_____].latency = 4;
    id_to_attr[ROTM_____].stall   = 0;
    id_to_attr[ROTMI____].name    = "rotmi";
    id_to_attr[ROTMI____].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTMI____].pipe    = PIPE_EVEN;
    id_to_attr[ROTMI____].latency = 4;
    id_to_attr[ROTMI____].stall   = 0;
    id_to_attr[ROTQMBY__].name    = "rotqmby";
    id_to_attr[ROTQMBY__].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTQMBY__].pipe    = PIPE_ODD;
    id_to_attr[ROTQMBY__].latency = 4;
    id_to_attr[ROTQMBY__].stall   = 0;
    id_to_attr[ROTQMBYI_].name    = "rotqmbyi";
    id_to_attr[ROTQMBYI_].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTQMBYI_].pipe    = PIPE_ODD;
    id_to_attr[ROTQMBYI_].latency = 4;
    id_to_attr[ROTQMBYI_].stall   = 0;
    id_to_attr[ROTQMBYBI].name    = "rotqmbybi";
    id_to_attr[ROTQMBYBI].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTQMBYBI].pipe    = PIPE_ODD;
    id_to_attr[ROTQMBYBI].latency = 4;
    id_to_attr[ROTQMBYBI].stall   = 0;
    id_to_attr[ROTQMBI__].name    = "rotqmbi";
    id_to_attr[ROTQMBI__].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTQMBI__].pipe    = PIPE_ODD;
    id_to_attr[ROTQMBI__].latency = 4;
    id_to_attr[ROTQMBI__].stall   = 0;
    id_to_attr[ROTQMBII_].name    = "rotqmbii";
    id_to_attr[ROTQMBII_].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTQMBII_].pipe    = PIPE_ODD;
    id_to_attr[ROTQMBII_].latency = 4;
    id_to_attr[ROTQMBII_].stall   = 0;
    id_to_attr[ROTMAH___].name    = "rotmah";
    id_to_attr[ROTMAH___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTMAH___].pipe    = PIPE_EVEN;
    id_to_attr[ROTMAH___].latency = 4;
    id_to_attr[ROTMAH___].stall   = 0;
    id_to_attr[ROTMAHI__].name    = "rotmahi";
    id_to_attr[ROTMAHI__].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTMAHI__].pipe    = PIPE_EVEN;
    id_to_attr[ROTMAHI__].latency = 4;
    id_to_attr[ROTMAHI__].stall   = 0;
    id_to_attr[ROTMA____].name    = "rotma";
    id_to_attr[ROTMA____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[ROTMA____].pipe    = PIPE_EVEN;
    id_to_attr[ROTMA____].latency = 4;
    id_to_attr[ROTMA____].stall   = 0;
    id_to_attr[ROTMAI___].name    = "rotmai";
    id_to_attr[ROTMAI___].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[ROTMAI___].pipe    = PIPE_EVEN;
    id_to_attr[ROTMAI___].latency = 4;
    id_to_attr[ROTMAI___].stall   = 0;
    id_to_attr[HEQ______].name    = "heq";
    id_to_attr[HEQ______].type    = IT_R_RA | IT_R_RB;
    id_to_attr[HEQ______].pipe    = PIPE_EVEN;
    id_to_attr[HEQ______].latency = 0;
    id_to_attr[HEQ______].stall   = 0;
    id_to_attr[HEQI_____].name    = "heqi";
    id_to_attr[HEQI_____].type    = IT_R_RA | IT_R_I10;
    id_to_attr[HEQI_____].pipe    = PIPE_EVEN;
    id_to_attr[HEQI_____].latency = 0;
    id_to_attr[HEQI_____].stall   = 0;
    id_to_attr[HGT______].name    = "hgt";
    id_to_attr[HGT______].type    = IT_R_RA | IT_R_RB;
    id_to_attr[HGT______].pipe    = PIPE_EVEN;
    id_to_attr[HGT______].latency = 0;
    id_to_attr[HGT______].stall   = 0;
    id_to_attr[HGTI_____].name    = "hgti";
    id_to_attr[HGTI_____].type    = IT_R_RA | IT_R_I10;
    id_to_attr[HGTI_____].pipe    = PIPE_EVEN;
    id_to_attr[HGTI_____].latency = 0;
    id_to_attr[HGTI_____].stall   = 0;
    id_to_attr[HLGT_____].name    = "hlgt";
    id_to_attr[HLGT_____].type    = IT_R_RA | IT_R_RB;
    id_to_attr[HLGT_____].pipe    = PIPE_EVEN;
    id_to_attr[HLGT_____].latency = 0;
    id_to_attr[HLGT_____].stall   = 0;
    id_to_attr[HLGTI____].name    = "hlgti";
    id_to_attr[HLGTI____].type    = IT_R_RA | IT_R_I10;
    id_to_attr[HLGTI____].pipe    = PIPE_EVEN;
    id_to_attr[HLGTI____].latency = 0;
    id_to_attr[HLGTI____].stall   = 0;
    id_to_attr[CEQB_____].name    = "ceqb";
    id_to_attr[CEQB_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CEQB_____].pipe    = PIPE_EVEN;
    id_to_attr[CEQB_____].latency = 2;
    id_to_attr[CEQB_____].stall   = 0;
    id_to_attr[CEQBI____].name    = "ceqbi";
    id_to_attr[CEQBI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CEQBI____].pipe    = PIPE_EVEN;
    id_to_attr[CEQBI____].latency = 2;
    id_to_attr[CEQBI____].stall   = 0;
    id_to_attr[CEQH_____].name    = "ceqh";
    id_to_attr[CEQH_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CEQH_____].pipe    = PIPE_EVEN;
    id_to_attr[CEQH_____].latency = 2;
    id_to_attr[CEQH_____].stall   = 0;
    id_to_attr[CEQHI____].name    = "ceqhi";
    id_to_attr[CEQHI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CEQHI____].pipe    = PIPE_EVEN;
    id_to_attr[CEQHI____].latency = 2;
    id_to_attr[CEQHI____].stall   = 0;
    id_to_attr[CEQ______].name    = "ceq";
    id_to_attr[CEQ______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CEQ______].pipe    = PIPE_EVEN;
    id_to_attr[CEQ______].latency = 2;
    id_to_attr[CEQ______].stall   = 0;
    id_to_attr[CEQI_____].name    = "ceqi";
    id_to_attr[CEQI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CEQI_____].pipe    = PIPE_EVEN;
    id_to_attr[CEQI_____].latency = 2;
    id_to_attr[CEQI_____].stall   = 0;
    id_to_attr[CGTB_____].name    = "cgtb";
    id_to_attr[CGTB_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CGTB_____].pipe    = PIPE_EVEN;
    id_to_attr[CGTB_____].latency = 2;
    id_to_attr[CGTB_____].stall   = 0;
    id_to_attr[CGTBI____].name    = "cgtbi";
    id_to_attr[CGTBI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CGTBI____].pipe    = PIPE_EVEN;
    id_to_attr[CGTBI____].latency = 2;
    id_to_attr[CGTBI____].stall   = 0;
    id_to_attr[CGTH_____].name    = "cgth";
    id_to_attr[CGTH_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CGTH_____].pipe    = PIPE_EVEN;
    id_to_attr[CGTH_____].latency = 2;
    id_to_attr[CGTH_____].stall   = 0;
    id_to_attr[CGTHI____].name    = "cgthi";
    id_to_attr[CGTHI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CGTHI____].pipe    = PIPE_EVEN;
    id_to_attr[CGTHI____].latency = 2;
    id_to_attr[CGTHI____].stall   = 0;
    id_to_attr[CGT______].name    = "cgt";
    id_to_attr[CGT______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CGT______].pipe    = PIPE_EVEN;
    id_to_attr[CGT______].latency = 2;
    id_to_attr[CGT______].stall   = 0;
    id_to_attr[CGTI_____].name    = "cgti";
    id_to_attr[CGTI_____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CGTI_____].pipe    = PIPE_EVEN;
    id_to_attr[CGTI_____].latency = 2;
    id_to_attr[CGTI_____].stall   = 0;
    id_to_attr[CLGTB____].name    = "clgtb";
    id_to_attr[CLGTB____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CLGTB____].pipe    = PIPE_EVEN;
    id_to_attr[CLGTB____].latency = 2;
    id_to_attr[CLGTB____].stall   = 0;
    id_to_attr[CLGTBI___].name    = "clgtbi";
    id_to_attr[CLGTBI___].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CLGTBI___].pipe    = PIPE_EVEN;
    id_to_attr[CLGTBI___].latency = 2;
    id_to_attr[CLGTBI___].stall   = 0;
    id_to_attr[CLGTH____].name    = "clgth";
    id_to_attr[CLGTH____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CLGTH____].pipe    = PIPE_EVEN;
    id_to_attr[CLGTH____].latency = 2;
    id_to_attr[CLGTH____].stall   = 0;
    id_to_attr[CLGTHI___].name    = "clgthi";
    id_to_attr[CLGTHI___].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CLGTHI___].pipe    = PIPE_EVEN;
    id_to_attr[CLGTHI___].latency = 2;
    id_to_attr[CLGTHI___].stall   = 0;
    id_to_attr[CLGT_____].name    = "clgt";
    id_to_attr[CLGT_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[CLGT_____].pipe    = PIPE_EVEN;
    id_to_attr[CLGT_____].latency = 2;
    id_to_attr[CLGT_____].stall   = 0;
    id_to_attr[CLGTI____].name    = "clgti";
    id_to_attr[CLGTI____].type    = IT_W_RT | IT_R_RA | IT_R_I10;
    id_to_attr[CLGTI____].pipe    = PIPE_EVEN;
    id_to_attr[CLGTI____].latency = 2;
    id_to_attr[CLGTI____].stall   = 0;
    id_to_attr[BR_______].name    = "br";
    id_to_attr[BR_______].type    = IT_R_I16;
    id_to_attr[BR_______].pipe    = PIPE_ODD;
    id_to_attr[BR_______].latency = 0;
    id_to_attr[BR_______].stall   = 0;
    id_to_attr[BRA______].name    = "bra";
    id_to_attr[BRA______].type    = IT_R_I16;
    id_to_attr[BRA______].pipe    = PIPE_ODD;
    id_to_attr[BRA______].latency = 0;
    id_to_attr[BRA______].stall   = 0;
    id_to_attr[BRSL_____].name    = "brsl";
    id_to_attr[BRSL_____].type    = IT_W_RT | IT_R_I16;
    id_to_attr[BRSL_____].pipe    = PIPE_ODD;
    id_to_attr[BRSL_____].latency = 0;
    id_to_attr[BRSL_____].stall   = 0;
    id_to_attr[BRASL____].name    = "brasl";
    id_to_attr[BRASL____].type    = IT_W_RT | IT_R_I16;
    id_to_attr[BRASL____].pipe    = PIPE_ODD;
    id_to_attr[BRASL____].latency = 0;
    id_to_attr[BRASL____].stall   = 0;
    id_to_attr[BI_______].name    = "bi";
    id_to_attr[BI_______].type    = IT_R_RA;
    id_to_attr[BI_______].pipe    = PIPE_ODD;
    id_to_attr[BI_______].latency = 0;
    id_to_attr[BI_______].stall   = 0;
    id_to_attr[IRET_____].name    = "iret";
    id_to_attr[IRET_____].type    = IT_R_RA;
    id_to_attr[IRET_____].pipe    = PIPE_ODD;
    id_to_attr[IRET_____].latency = 0;
    id_to_attr[IRET_____].stall   = 0;
    id_to_attr[BISLED___].name    = "bisled";
    id_to_attr[BISLED___].type    = IT_W_RT | IT_R_RA;
    id_to_attr[BISLED___].pipe    = PIPE_ODD;
    id_to_attr[BISLED___].latency = 4;
    id_to_attr[BISLED___].stall   = 0;
    id_to_attr[BISL_____].name    = "bisl";
    id_to_attr[BISL_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[BISL_____].pipe    = PIPE_ODD;
    id_to_attr[BISL_____].latency = 4;
    id_to_attr[BISL_____].stall   = 0;
    id_to_attr[BRNZ_____].name    = "brnz";
    id_to_attr[BRNZ_____].type    = IT_R_RT | IT_R_I16;
    id_to_attr[BRNZ_____].pipe    = PIPE_ODD;
    id_to_attr[BRNZ_____].latency = 0;
    id_to_attr[BRNZ_____].stall   = 0;
    id_to_attr[BRZ______].name    = "brz";
    id_to_attr[BRZ______].type    = IT_R_RT | IT_R_I16;
    id_to_attr[BRZ______].pipe    = PIPE_ODD;
    id_to_attr[BRZ______].latency = 0;
    id_to_attr[BRZ______].stall   = 0;
    id_to_attr[BRHNZ____].name    = "brhnz";
    id_to_attr[BRHNZ____].type    = IT_R_RT | IT_R_I16;
    id_to_attr[BRHNZ____].pipe    = PIPE_ODD;
    id_to_attr[BRHNZ____].latency = 0;
    id_to_attr[BRHNZ____].stall   = 0;
    id_to_attr[BRHZ_____].name    = "brhz";
    id_to_attr[BRHZ_____].type    = IT_R_RT | IT_R_I16;
    id_to_attr[BRHZ_____].pipe    = PIPE_ODD;
    id_to_attr[BRHZ_____].latency = 0;
    id_to_attr[BRHZ_____].stall   = 0;
    id_to_attr[BIZ______].name    = "biz";
    id_to_attr[BIZ______].type    = IT_R_RA | IT_R_RT;;
    id_to_attr[BIZ______].pipe    = PIPE_ODD;
    id_to_attr[BIZ______].latency = 0;
    id_to_attr[BIZ______].stall   = 0;
    id_to_attr[BINZ_____].name    = "binz";
    id_to_attr[BINZ_____].type    = IT_R_RA | IT_R_RT;;
    id_to_attr[BINZ_____].pipe    = PIPE_ODD;
    id_to_attr[BINZ_____].latency = 0;
    id_to_attr[BINZ_____].stall   = 0;
    id_to_attr[BIHZ_____].name    = "bihz";
    id_to_attr[BIHZ_____].type    = IT_R_RA | IT_R_RT;;
    id_to_attr[BIHZ_____].pipe    = PIPE_ODD;
    id_to_attr[BIHZ_____].latency = 0;
    id_to_attr[BIHZ_____].stall   = 0;
    id_to_attr[BIHNZ____].name    = "bihnz";
    id_to_attr[BIHNZ____].type    = IT_R_RA | IT_R_RT;;
    id_to_attr[BIHNZ____].pipe    = PIPE_ODD;
    id_to_attr[BIHNZ____].latency = 0;
    id_to_attr[BIHNZ____].stall   = 0;
    id_to_attr[HBR______].name    = "hbr";
    id_to_attr[HBR______].type    = IT_R_RA | IT_R_RO1;
    id_to_attr[HBR______].pipe    = PIPE_ODD;
    id_to_attr[HBR______].latency = 15;
    id_to_attr[HBR______].stall   = 0;
    id_to_attr[HBRA_____].name    = "hbra";
    id_to_attr[HBRA_____].type    = IT_R_I16 | IT_R_RO2;
    id_to_attr[HBRA_____].pipe    = PIPE_ODD;
    id_to_attr[HBRA_____].latency = 15;
    id_to_attr[HBRA_____].stall   = 0;
    id_to_attr[HBRR_____].name    = "hbrr";
    id_to_attr[HBRR_____].type    = IT_R_I16 | IT_R_RO2;
    id_to_attr[HBRR_____].pipe    = PIPE_ODD;
    id_to_attr[HBRR_____].latency = 15;
    id_to_attr[HBRR_____].stall   = 0;
    id_to_attr[FA_______].name    = "fa";
    id_to_attr[FA_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FA_______].pipe    = PIPE_EVEN;
    id_to_attr[FA_______].latency = 6;
    id_to_attr[FA_______].stall   = 0;
    id_to_attr[DFA______].name    = "dfa";
    id_to_attr[DFA______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[DFA______].pipe    = PIPE_EVEN;
    id_to_attr[DFA______].latency = 7;
    id_to_attr[DFA______].stall   = 6;
    id_to_attr[FS_______].name    = "fs";
    id_to_attr[FS_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FS_______].pipe    = PIPE_EVEN;
    id_to_attr[FS_______].latency = 6;
    id_to_attr[FS_______].stall   = 0;
    id_to_attr[DFS______].name    = "dfs";
    id_to_attr[DFS______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[DFS______].pipe    = PIPE_EVEN;
    id_to_attr[DFS______].latency = 7;
    id_to_attr[DFS______].stall   = 6;
    id_to_attr[FM_______].name    = "fm";
    id_to_attr[FM_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FM_______].pipe    = PIPE_EVEN;
    id_to_attr[FM_______].latency = 6;
    id_to_attr[FM_______].stall   = 0;
    id_to_attr[DFM______].name    = "dfm";
    id_to_attr[DFM______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[DFM______].pipe    = PIPE_EVEN;
    id_to_attr[DFM______].latency = 7;
    id_to_attr[DFM______].stall   = 6;
    id_to_attr[FMA______].name    = "fma";
    id_to_attr[FMA______].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RC;
    id_to_attr[FMA______].pipe    = PIPE_EVEN;
    id_to_attr[FMA______].latency = 6;
    id_to_attr[FMA______].stall   = 0;
    id_to_attr[DFMA_____].name    = "dfma";
    id_to_attr[DFMA_____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[DFMA_____].pipe    = PIPE_EVEN;
    id_to_attr[DFMA_____].latency = 7;
    id_to_attr[DFMA_____].stall   = 6;
    id_to_attr[FNMS_____].name    = "fnms";
    id_to_attr[FNMS_____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RC;
    id_to_attr[FNMS_____].pipe    = PIPE_EVEN;
    id_to_attr[FNMS_____].latency = 6;
    id_to_attr[FNMS_____].stall   = 0;
    id_to_attr[DFNMS____].name    = "dfnms";
    id_to_attr[DFNMS____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[DFNMS____].pipe    = PIPE_EVEN;
    id_to_attr[DFNMS____].latency = 7;
    id_to_attr[DFNMS____].stall   = 6;
    id_to_attr[FMS______].name    = "fms";
    id_to_attr[FMS______].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RC;
    id_to_attr[FMS______].pipe    = PIPE_EVEN;
    id_to_attr[FMS______].latency = 6;
    id_to_attr[FMS______].stall   = 0;
    id_to_attr[DFMS_____].name    = "dfms";
    id_to_attr[DFMS_____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[DFMS_____].pipe    = PIPE_EVEN;
    id_to_attr[DFMS_____].latency = 7;
    id_to_attr[DFMS_____].stall   = 6;
    id_to_attr[DFNMA____].name    = "dfnma";
    id_to_attr[DFNMA____].type    = IT_W_RT | IT_R_RA | IT_R_RB | IT_R_RT;
    id_to_attr[DFNMA____].pipe    = PIPE_EVEN;
    id_to_attr[DFNMA____].latency = 7;
    id_to_attr[DFNMA____].stall   = 6;
    id_to_attr[FREST____].name    = "frest";
    id_to_attr[FREST____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[FREST____].pipe    = PIPE_ODD;
    id_to_attr[FREST____].latency = 4;
    id_to_attr[FREST____].stall   = 0;
    id_to_attr[FRSQEST__].name    = "frsqest";
    id_to_attr[FRSQEST__].type    = IT_W_RT | IT_R_RA;
    id_to_attr[FRSQEST__].pipe    = PIPE_ODD;
    id_to_attr[FRSQEST__].latency = 4;
    id_to_attr[FRSQEST__].stall   = 0;
    id_to_attr[FI_______].name    = "fi";
    id_to_attr[FI_______].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FI_______].pipe    = PIPE_EVEN;
    id_to_attr[FI_______].latency = 7;
    id_to_attr[FI_______].stall   = 0;
    id_to_attr[CSFLT____].name    = "csflt";
    id_to_attr[CSFLT____].type    = IT_W_RT | IT_R_RA | IT_R_I8;
    id_to_attr[CSFLT____].pipe    = PIPE_EVEN;
    id_to_attr[CSFLT____].latency = 7;
    id_to_attr[CSFLT____].stall   = 0;
    id_to_attr[CFLTS____].name    = "cflts";
    id_to_attr[CFLTS____].type    = IT_W_RT | IT_R_RA | IT_R_I8;
    id_to_attr[CFLTS____].pipe    = PIPE_EVEN;
    id_to_attr[CFLTS____].latency = 7;
    id_to_attr[CFLTS____].stall   = 0;
    id_to_attr[CUFLT____].name    = "cuflt";
    id_to_attr[CUFLT____].type    = IT_W_RT | IT_R_RA | IT_R_I8;
    id_to_attr[CUFLT____].pipe    = PIPE_EVEN;
    id_to_attr[CUFLT____].latency = 7;
    id_to_attr[CUFLT____].stall   = 0;
    id_to_attr[CFLTU____].name    = "cfltu";
    id_to_attr[CFLTU____].type    = IT_W_RT | IT_R_RA | IT_R_I8;
    id_to_attr[CFLTU____].pipe    = PIPE_EVEN;
    id_to_attr[CFLTU____].latency = 7;
    id_to_attr[CFLTU____].stall   = 0;
    id_to_attr[FRDS_____].name    = "frds";
    id_to_attr[FRDS_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[FRDS_____].pipe    = PIPE_EVEN;
    id_to_attr[FRDS_____].latency = 7;
    id_to_attr[FRDS_____].stall   = 6;
    id_to_attr[FESD_____].name    = "fesd";
    id_to_attr[FESD_____].type    = IT_W_RT | IT_R_RA;
    id_to_attr[FESD_____].pipe    = PIPE_EVEN;
    id_to_attr[FESD_____].latency = 7;
    id_to_attr[FESD_____].stall   = 6;
    id_to_attr[DFCEQ____].name    = "dfceq";
    id_to_attr[DFCEQ____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[DFCEQ____].pipe    = PIPE_EVEN;
    id_to_attr[DFCEQ____].latency = 7;
    id_to_attr[DFCEQ____].stall   = 6;
    id_to_attr[DFCMEQ___].name    = "dfcmeq";
    id_to_attr[DFCMEQ___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[DFCMEQ___].pipe    = PIPE_EVEN;
    id_to_attr[DFCMEQ___].latency = 7;
    id_to_attr[DFCMEQ___].stall   = 6;
    id_to_attr[DFCGT____].name    = "dfcgt";
    id_to_attr[DFCGT____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[DFCGT____].pipe    = PIPE_EVEN;
    id_to_attr[DFCGT____].latency = 7;
    id_to_attr[DFCGT____].stall   = 6;
    id_to_attr[DFCMGT___].name    = "dfcmgt";
    id_to_attr[DFCMGT___].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[DFCMGT___].pipe    = PIPE_EVEN;
    id_to_attr[DFCMGT___].latency = 7;
    id_to_attr[DFCMGT___].stall   = 6;
    id_to_attr[DFTSV____].name    = "dftsv";
    id_to_attr[DFTSV____].type    = IT_W_RT | IT_R_RA | IT_R_I7;
    id_to_attr[DFTSV____].pipe    = PIPE_EVEN;
    id_to_attr[DFTSV____].latency = 7;
    id_to_attr[DFTSV____].stall   = 6;
    id_to_attr[FCEQ_____].name    = "fceq";
    id_to_attr[FCEQ_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FCEQ_____].pipe    = PIPE_EVEN;
    id_to_attr[FCEQ_____].latency = 2;
    id_to_attr[FCEQ_____].stall   = 0;
    id_to_attr[FCMEQ____].name    = "fcmeq";
    id_to_attr[FCMEQ____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FCMEQ____].pipe    = PIPE_EVEN;
    id_to_attr[FCMEQ____].latency = 2;
    id_to_attr[FCMEQ____].stall   = 0;
    id_to_attr[FCGT_____].name    = "fcgt";
    id_to_attr[FCGT_____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FCGT_____].pipe    = PIPE_EVEN;
    id_to_attr[FCGT_____].latency = 2;
    id_to_attr[FCGT_____].stall   = 0;
    id_to_attr[FCMGT____].name    = "fcmgt";
    id_to_attr[FCMGT____].type    = IT_W_RT | IT_R_RA | IT_R_RB;
    id_to_attr[FCMGT____].pipe    = PIPE_EVEN;
    id_to_attr[FCMGT____].latency = 2;
    id_to_attr[FCMGT____].stall   = 0;
    id_to_attr[FSCRWR___].name    = "fscrwr";
    id_to_attr[FSCRWR___].type    = IT_R_RA;
    id_to_attr[FSCRWR___].pipe    = PIPE_EVEN;
    id_to_attr[FSCRWR___].latency = 7;
    id_to_attr[FSCRWR___].stall   = 0;
    id_to_attr[FSCRRD___].name    = "fscrrd";
    id_to_attr[FSCRRD___].type    = IT_W_RT;
    id_to_attr[FSCRRD___].pipe    = PIPE_EVEN;
    id_to_attr[FSCRRD___].latency = 7;
    id_to_attr[FSCRRD___].stall   = 6;
    id_to_attr[STOP_____].name    = "stop";
    id_to_attr[STOP_____].type    = IT_NONE;
    id_to_attr[STOP_____].pipe    = PIPE_ODD;
    id_to_attr[STOP_____].latency = 4;
    id_to_attr[STOP_____].stall   = 0;
    id_to_attr[STOPD____].name    = "stopd";
    id_to_attr[STOPD____].type    = IT_NONE;
    id_to_attr[STOPD____].pipe    = PIPE_ODD;
    id_to_attr[STOPD____].latency = 4;
    id_to_attr[STOPD____].stall   = 0;
    id_to_attr[LNOP_____].name    = "lnop";
    id_to_attr[LNOP_____].type    = IT_NONE;
    id_to_attr[LNOP_____].pipe    = PIPE_ODD;
    id_to_attr[LNOP_____].latency = 0;
    id_to_attr[LNOP_____].stall   = 0;
    id_to_attr[NOP______].name    = "nop";
    id_to_attr[NOP______].type    = IT_NONE;
    id_to_attr[NOP______].pipe    = PIPE_EVEN;
    id_to_attr[NOP______].latency = 0;
    id_to_attr[NOP______].stall   = 0;
    id_to_attr[SYNC_____].name    = "sync";
    id_to_attr[SYNC_____].type    = IT_NONE;
    id_to_attr[SYNC_____].pipe    = PIPE_ODD;
    id_to_attr[SYNC_____].latency = 0;
    id_to_attr[SYNC_____].stall   = 0;
    id_to_attr[DSYNC____].name    = "dsync";
    id_to_attr[DSYNC____].type    = IT_NONE;
    id_to_attr[DSYNC____].pipe    = PIPE_ODD;
    id_to_attr[DSYNC____].latency = 0;
    id_to_attr[DSYNC____].stall   = 0;
    id_to_attr[MFSPR____].name    = "mfspr";
    id_to_attr[MFSPR____].type    = IT_W_RT;
    id_to_attr[MFSPR____].pipe    = PIPE_ODD;
    id_to_attr[MFSPR____].latency = 6;
    id_to_attr[MFSPR____].stall   = 0;
    id_to_attr[MTSPR____].name    = "mtspr";
    id_to_attr[MTSPR____].type    = IT_R_RT;
    id_to_attr[MTSPR____].pipe    = PIPE_ODD;
    id_to_attr[MTSPR____].latency = 6;
    id_to_attr[MTSPR____].stall   = 0;
    id_to_attr[RDCH_____].name    = "rdch";
    id_to_attr[RDCH_____].type    = IT_W_RT | IT_R_CA;
    id_to_attr[RDCH_____].pipe    = PIPE_ODD;
    id_to_attr[RDCH_____].latency = 6;
    id_to_attr[RDCH_____].stall   = 0;
    id_to_attr[WRCH_____].name    = "wrch";
    id_to_attr[WRCH_____].type    = IT_R_RT | IT_R_CA;
    id_to_attr[WRCH_____].pipe    = PIPE_ODD;
    id_to_attr[WRCH_____].latency = 0;
    id_to_attr[WRCH_____].stall   = 0;
    id_to_attr[RCHCNT___].name    = "rchcnt";
    id_to_attr[RCHCNT___].type    = IT_W_RT | IT_R_CA;
    id_to_attr[RCHCNT___].pipe    = PIPE_ODD;
    id_to_attr[RCHCNT___].latency = 6;
    id_to_attr[RCHCNT___].stall   = 0;
}

/**********************************************************************/ 
int SpuInstinfo::getinstid(int op)
{
    return op_to_id[op];
}

/**********************************************************************/
spuinstattr_t SpuInstinfo::getinstattr(int id)
{
    return id_to_attr[id];
}

/**********************************************************************/
char *SpuInstinfo::getopname(int id)
{
    return id_to_attr[id].name;
}
    
/**********************************************************************/ 
SpuInstinfo::SpuInstinfo(Chip *chip)
{
    chip->iinfo = this;
    for (uint i = 0; i < IINFO_OP_NUM; i++)
        op_to_id[i] = predecode(i);
    for (uint i = 0; i < IINFO_ID_NUM; i++) {
        id_to_attr[i].type = 0;
        id_to_attr[i].pipe = 0;
        id_to_attr[i].latency = 0;
        id_to_attr[i].stall = 0;
        id_to_attr[i].name = "undefined";
    }
    setattr();
}

/**********************************************************************/
